/**
 * math.js
 * https://github.com/josdejong/mathjs
 *
 * Math.js is an extensive math library for JavaScript and Node.js,
 * It features real and complex numbers, units, matrices, a large set of
 * mathematical functions, and a flexible expression parser.
 *
 * @version 0.7.0
 * @date    2013-04-20
 *
 * @license
 * Copyright (C) 2013 Jos de Jong <wjosdejong@gmail.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy
 * of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
(function(){function e(e){return e instanceof Boolean||"boolean"==typeof e}function t(e,n){if(this.constructor!=t)throw new SyntaxError("Complex constructor must be called with the new operator");if(null!=e&&!v(e)||null!=n&&!v(n))throw new TypeError("Two numbers or a single string expected in Complex constructor");this.re=e||0,this.im=n||0}function n(e){if(this.constructor!=n)throw new SyntaxError("Matrix constructor must be called with the new operator");if(e instanceof n||e instanceof w)this._data=e.toArray();else if(e instanceof Array)this._data=e;else{if(null!=e)throw new TypeError("Unsupported type of data ("+Z.typeof(e)+")");this._data=[]}this._size=K.size(this._data)}function r(e,t){if(!v(e)||!g(e))throw new TypeError("Index must be an integer (value: "+e+")");if(1>e)throw new RangeError("Index out of range ("+e+" < 1)");if(t&&e>t)throw new RangeError("Index out of range ("+e+" > "+t+")")}function i(e,t){return r(t,e.length),e[t-1]}function a(e,t){return t.forEach(function(t){e=i(e,t)}),Z.clone(e)}function o(e,t){var n=t[0];return n.map?n.map(function(t){return i(e,t)}):[i(e,n)]}function f(e,t){var n=t[0],r=t[1];if(n.map)return r.map?n.map(function(t){var n=i(e,t);return r.map(function(e){return i(n,e)})}):n.map(function(t){return[i(i(e,t),r)]});if(r.map){var a=i(e,n);return[r.map(function(e){return i(a,e)})]}return[[i(i(e,n),r)]]}function s(e,t,n){var r=n==t.length-1,a=t[n],o=function(a){var o=i(e,a);return r?o:s(o,t,n+1)};return a.map?a.map(o):[o(a)]}function u(e,t,n){if(r(t),n instanceof Array)throw new TypeError("Dimension mismatch, value expected instead of array");e[t-1]=n}function c(e,t,n,i){var a=!1;n.length>t.length&&(a=!0);for(var o=0;n.length>o;o++){var f=n[o];r(f),(null==t[o]||f>t[o])&&(t[o]=f,a=!0)}a&&K.resize(e,t,0);var s=t.length;n.forEach(function(t,n){s-1>n?e=e[t-1]:e[t-1]=i})}function l(e,t,n,i){var a=n[0];r(a),a>t[0]&&K.resize(e,[a],0),e[a-1]=i}function h(e,t,n,i){var a=n[0],o=n[1];r(a),r(o);var f=!1;a>(t[0]||0)&&(t[0]=a,f=!0),o>(t[1]||0)&&(t[1]=o,f=!0),f&&K.resize(e,t,0),e[a-1][o-1]=i}function m(e,t,n,r,i){var a=r==n.length-1,o=n[r],f=function(o,f){if(a)u(e,o,i[f]),e.length>(t[r]||0)&&(t[r]=e.length);else{var s=e[o-1];s instanceof Array||(e[o-1]=s=[s],e.length>(t[r]||0)&&(t[r]=e.length)),m(s,t,n,r+1,i[f])}};if(o.map){var s=o.size&&o.size()||o.length;if(s!=i.length)throw new RangeError("Dimensions mismatch ("+s+" != "+i.length+")");o.map(f)}else f(o,0)}function p(e){for(var t=0,n=e.length;n>t;t++){var r=e[t];r instanceof Array?p(r):void 0==r&&(e[t]=0)}}function v(e){return e instanceof Number||"number"==typeof e}function g(e){return e==Math.round(e)}function w(e,t,n){if(this.constructor!=w)throw new SyntaxError("Range constructor must be called with the new operator");if(null!=e&&!v(e))throw new TypeError("Parameter start must be a number");if(null!=n&&!v(n))throw new TypeError("Parameter end must be a number");if(null!=t&&!v(t))throw new TypeError("Parameter step must be a number");this.start=null!=e?e:0,this.end=null!=n?n:0,this.step=null!=t?t:1}function d(e,t){var n=Z.type.Selector,r=Array.prototype.slice;n.prototype[e]="function"==typeof t?function(){var e=[this.value].concat(r.call(arguments,0));return new n(t.apply(this,e))}:new n(t)}function y(e){return e instanceof String||"string"==typeof e}function E(e,t){if(this.constructor!=E)throw Error("Unit constructor must be called with the new operator");if(null!=e&&!v(e))throw Error("First parameter in Unit constructor must be a number");if(null!=t&&!y(t))throw Error("Second parameter in Unit constructor must be a string");if(null!=t){var n=N(t);if(!n)throw Error('String "'+t+'" is no unit');this.unit=n.unit,this.prefix=n.prefix}else this.unit=E.UNIT_NONE,this.prefix=E.PREFIX_NONE;null!=e?(this.value=this._normalize(e),this.fixPrefix=!1):(this.value=null,this.fixPrefix=!0)}function N(e){for(var t=E.UNITS,n=0,r=t.length;r>n;n++){var i=t[n];if(K.endsWith(e,i.name)){var a=e.length-i.name.length,o=e.substring(0,a),f=i.prefixes[o];if(void 0!==f)return{unit:i,prefix:f}}}return null}function O(e,t){var n=void 0;if(2==arguments.length){var r=Z.typeof(t);n="Function "+e+"("+r+") not supported"}else if(arguments.length>2){for(var i=[],a=1;arguments.length>a;a++)i.push(Z.typeof(arguments[a]));n="Function "+e+"("+i.join(", ")+") not supported"}else n="Unsupported parameter in function "+e;return new TypeError(n)}function x(e,t,n,r){var i="Wrong number of arguments in function "+e+" ("+t+" provided, "+n+(void 0!=r?"-"+r:"")+" expected)";return new SyntaxError(i)}function b(){}function M(e,t,n){this.name=e,this.fn=t,this.params=n}function A(e){this.value=e}function S(e){this.nodes=e||[]}function T(){this.params=[],this.visible=[]}function U(e,t,n,r){this.name=e,this.params=t,this.expr=n,this.result=r}function L(e,t){this.object=e,this.params=t}function R(e,t,n,r,i){this.name=e,this.variables=n,this.values=[];for(var a=0,o=this.variables.length;o>a;a++)this.values[a]=function(){var e=function(){return e.value};return e.value=void 0,e}();this.def=this.createFunction(e,t,n,r),this.result=i}function _(e,n){var r=n.re*n.re+n.im*n.im;return new t((e.re*n.re+e.im*n.im)/r,(e.im*n.re-e.re*n.im)/r)}function z(e,n){return new t(e.re*n.re-e.im*n.im,e.re*n.im+e.im*n.re)}function q(e,t){var n=Z.log(e),r=Z.multiply(n,t);return Z.exp(r)}function I(e,t){if(t){var n=Math.pow(10,t);return Math.round(e*n)/n}return Math.round(e)}function P(e,t,n,r){if(n>r){if(e.length!=t.length)throw Error("Dimensions mismatch ("+e.length+" != "+t.length+")");for(var i=[],a=0;e.length>a;a++)i[a]=P(e[a],t[a],n,r+1);return i}return e.concat(t)}function C(e,t,n){var r=Z.multiply,i=Z.subtract;if(1==t)return e[0][0];if(2==t)return i(r(e[0][0],e[1][1]),r(e[1][0],e[0][1]));for(var a=0,o=0;n>o;o++){var f=B(e,t,n,0,o);a+=r(r((o+1)%2+(o+1)%2-1,e[0][o]),C(f,t-1,n-1))}return a}function B(e,t,n,r,i){for(var a,o=[],f=0;t>f;f++)if(f!=r){a=o[f-(f>r)]=[];for(var s=0;n>s;s++)s!=i&&(a[s-(s>i)]=e[f][s])}return o}function G(e,t,n){var r,i,a,o,f,s=Z.add,u=Z.unaryminus,c=Z.multiply,l=Z.divide;if(1==t){if(o=e[0][0],0==o)throw Error("Cannot calculate inverse, determinant is zero");return[[l(1,o)]]}if(2==t){var h=Z.det(e);if(0==h)throw Error("Cannot calculate inverse, determinant is zero");return[[l(e[1][1],h),l(u(e[0][1]),h)],[l(u(e[1][0]),h),l(e[0][0],h)]]}var m=e.concat();for(r=0;t>r;r++)m[r]=m[r].concat();for(var p=Z.eye(t).valueOf(),v=0;n>v;v++){for(r=v;t>r&&0==m[r][v];)r++;if(r==t||0==m[r][v])throw Error("Cannot calculate inverse, determinant is zero");r!=v&&(f=m[v],m[v]=m[r],m[r]=f,f=p[v],p[v]=p[r],p[r]=f);var g=m[v],w=p[v];for(r=0;t>r;r++){var d=m[r],y=p[r];if(r!=v){if(0!=d[v]){for(a=l(u(d[v]),g[v]),i=v;n>i;i++)d[i]=s(d[i],c(a,g[i]));for(i=0;n>i;i++)y[i]=s(y[i],c(a,w[i]))}}else{for(a=g[v],i=v;n>i;i++)d[i]=l(d[i],a);for(i=0;n>i;i++)y[i]=l(y[i],a)}}}return p}function D(e){if(1==e.length)return D(e[0]);for(var t=0,n=e.length;n>t;t++){var r=e[t];r instanceof Array&&(e[t]=D(r))}return e}function k(e){for(var t=Z.larger,n=e[0],r=1,i=e.length;i>r;r++){var a=e[r];t(a,n)&&(n=a)}return n}function V(e,t,n){for(var r=Z.larger,i=[],a=0;n>a;a++){for(var o=e[0][a],f=1;t>f;f++){var s=e[f][a];r(s,o)&&(o=s)}i[a]=o}return i}function F(e){for(var t=Z.smaller,n=e[0],r=1,i=e.length;i>r;r++){var a=e[r];t(a,n)&&(n=a)}return n}function j(e,t,n){for(var r=Z.smaller,i=[],a=0;n>a;a++){for(var o=e[0][a],f=1;t>f;f++){var s=e[f][a];r(s,o)&&(o=s)}i[a]=o}return i}function H(e,t){Z[e]=t,d(e,t)}function Y(e){return"function"==typeof e||v(e)||y(e)||e instanceof t||e instanceof E}function W(e,t){throw Error('Constructor "'+e+'" has been replaced by '+'constructor method "'+t+'" in math.js v0.5.0')}var Z={type:{},expr:{node:{}},options:{precision:5}};"undefined"!=typeof module&&module.exports!==void 0&&(module.exports=Z),"undefined"!=typeof exports&&(exports=Z),"undefined"!=typeof require&&"undefined"!=typeof define&&define(function(){return Z}),"undefined"!=typeof window&&(window.math=Z);var K=function(){function e(t){if(t instanceof Array){var n=t.length;if(n){var r=e(t[0]);return 0==r[0]?[0].concat(r):[n].concat(r)}return[n]}return[]}function t(e,n,r){var i,a=e.length;if(a!=n[r])throw new RangeError("Dimension mismatch ("+a+" != "+n[r]+")");if(n.length-1>r){var o=r+1;for(i=0;a>i;i++){var f=e[i];if(!(f instanceof Array))throw new RangeError("Dimension mismatch ("+(n.length-1)+" < "+n.length+")");t(e[i],n,o)}}else for(i=0;a>i;i++)if(e[i]instanceof Array)throw new RangeError("Dimension mismatch ("+(n.length+1)+" > "+n.length+")")}function r(e,t,n){if(t.length-1>n){var i=e[0];if(1!=e.length||!(i instanceof Array))throw new RangeError("Dimension mismatch ("+e.length+" > 0)");r(i,t,n+1)}else if(e.length)throw new RangeError("Dimension mismatch ("+e.length+" > 0)")}function i(e,t,n,r){if(!(e instanceof Array))throw new TypeError("Array expected");var a=e.length,o=t[n];if(a!=o){if(o>e.length)for(var f=e.length;o>f;f++)e[f]=r?Z.clone(r):0;else e.length=t[n];a=e.length}if(t.length-1>n){var s=n+1;for(f=0;a>f;f++)u=e[f],u instanceof Array||(u=[u],e[f]=u),i(u,t,s,r)}else for(f=0;a>f;f++){for(var u=e[f];u instanceof Array;)u=u[0];e[f]=u}}var a={};a.formatNumber=function(e,t){if(1/0===e)return"Infinity";if(e===-1/0)return"-Infinity";if(0/0===e)return"NaN";var n=Math.abs(e);if(n>.001&&1e5>n||0==n)return a.toPrecision(e,t);var r=Math.round(Math.log(n)/Math.LN10),i=e/Math.pow(10,r);return a.toPrecision(i,t)+"e"+r},a.toPrecision=function(e,t){return void 0==t&&(t=Z.options.precision),e.toPrecision(t).replace(o,function(e,t,n){return e.substring(0,e.length-(t.length?0:1)-n.length)})};var o=/\.(\d*?)(0+)$/g;return a.formatArray=function(e){if(e instanceof Array){for(var t="[",n=e.length,r=0;n>r;r++)0!=r&&(t+=", "),t+=a.formatArray(e[r]);return t+="]"}return Z.format(e)},a.formatArray2d=function(e){var t="[",n=a.size(e);if(2!=n.length)throw new RangeError("Array must be two dimensional (size: "+a.formatArray(n)+")");for(var r=n[0],i=n[1],o=0;r>o;o++){0!=o&&(t+="; ");for(var f=e[o],s=0;i>s;s++){0!=s&&(t+=", ");var u=f[s];void 0!=u&&(t+=Z.format(u))}}return t+="]"},a.argsToArray=function(e){var t;if(0==e.length)t=[];else if(1==e.length)t=e[0],t instanceof n&&(t=t.toVector()),t instanceof w&&(t=t.valueOf()),t instanceof Array||(t=[t]);else{t=[];for(var r=0;e.length>r;r++)t[r]=e[r]}return t},a.endsWith=function(e,t){var n=e.length-t.length,r=e.length;return e.substring(n,r)===t},a.extend=function(e,t){for(var n in t)t.hasOwnProperty(n)&&(e[n]=t[n]);return e},a.randomUUID=function(){var e=function(){return Math.floor(65536*Math.random()).toString(16)};return e()+e()+"-"+e()+"-"+e()+"-"+e()+"-"+e()+e()+e()},a.map=function(e,t){if(e instanceof Array||e instanceof n||e instanceof w)return e.map(function(e){return t(e)});throw new TypeError("Array expected")},a.map2=function(e,t,r){var i,o,f;if(e instanceof n||t instanceof n)return new n(a.map2(e.valueOf(),t.valueOf(),r));if(e instanceof w||t instanceof w)return a.map2(e.valueOf(),t.valueOf(),r);if(e instanceof Array)if(t instanceof Array){if(e.length!=t.length)throw new RangeError("Dimension mismatch ("+e.length+" != "+t.length+")");for(i=[],o=e.length,f=0;o>f;f++)i[f]=r(e[f],t[f])}else for(i=[],o=e.length,f=0;o>f;f++)i[f]=r(e[f],t);else if(t instanceof Array)for(i=[],o=t.length,f=0;o>f;f++)i[f]=r(e,t[f]);else i=r(e,t);return i},a.forEach=function(e,t){if(e instanceof Array)e.forEach(t);else for(var n in e)e.hasOwnProperty(n)&&t(e[n],n,e)},a.mapObject=function(e,t){var n={};for(var r in e)e.hasOwnProperty(r)&&(n[r]=t(e[r]));return n},a.deepEqual=function(e,t){var n,r,i;if(e instanceof Array){if(!(t instanceof Array))return!1;for(r=0,i=e.length;i>r;r++)if(!a.deepEqual(e[r],t[r]))return!1;return!0}if(e instanceof Object){if(t instanceof Array||!(t instanceof Object))return!1;for(n in e)if(e.hasOwnProperty(n)&&!a.deepEqual(e[n],t[n]))return!1;for(n in t)if(t.hasOwnProperty(n)&&!a.deepEqual(e[n],t[n]))return!1;return!0}return e.valueOf()==t.valueOf()},a.size=function(t){var n=e(t);return a.validate(t,n),n},a.validate=function(e,n){var i=0==n.length;if(i){if(e instanceof Array)throw new RangeError("Dimension mismatch ("+e.length+" != 0)")}else{var o=-1!=n.indexOf(0);o?(n.forEach(function(e){if(0!=e)throw new RangeError("Invalid size, all dimensions must be either zero or non-zero (size: "+a.formatArray(n)+")")}),r(e,n,0)):t(e,n,0)}},a.resize=function(e,t,n){if(!(t instanceof Array))throw new TypeError("Size must be an array (size is "+Z.typeof(t)+")");t.forEach(function(e){if(!v(e)||!g(e)||0>e)throw new TypeError("Invalid size, must contain positive integers (size: "+a.formatArray(t)+")")});var r=-1!=t.indexOf(0);r&&t.forEach(function(e){if(0!=e)throw new RangeError("Invalid size, all dimensions must be either zero or non-zero (size: "+a.formatArray(t)+")")}),i(e,t,0,n)},Array.prototype.indexOf||(Array.prototype.indexOf=function(e){for(var t=0;this.length>t;t++)if(this[t]==e)return t;return-1}),Array.prototype.forEach||(Array.prototype.forEach=function(e,t){for(var n=0,r=this.length;r>n;++n)e.call(t||this,this[n],n,this)}),Array.prototype.map||(Array.prototype.map=function(e,t){var n,r,i;if(null==this)throw new TypeError(" this is null or not defined");var a=Object(this),o=a.length>>>0;if("function"!=typeof e)throw new TypeError(e+" is not a function");for(t&&(n=t),r=Array(o),i=0;o>i;){var f,s;i in a&&(f=a[i],s=e.call(n,f,i,a),r[i]=s),i++}return r}),Array.prototype.every||(Array.prototype.every=function(e){"use strict";if(null==this)throw new TypeError;var t=Object(this),n=t.length>>>0;if("function"!=typeof e)throw new TypeError;for(var r=arguments[1],i=0;n>i;i++)if(i in t&&!e.call(r,t[i],i,t))return!1;return!0}),Array.prototype.some||(Array.prototype.some=function(e){"use strict";if(null==this)throw new TypeError;var t=Object(this),n=t.length>>>0;if("function"!=typeof e)throw new TypeError;for(var r=arguments[1],i=0;n>i;i++)if(i in t&&e.call(r,t[i],i,t))return!0;return!1}),a}();Z.type.Complex=t,function(){function e(){for(;" "==c||"	"==c;)i()}function n(e){return e>="0"&&"9">=e||"."==e}function r(e){return e>="0"&&"9">=e}function i(){u++,c=s[u]}function a(e){u=e,c=s[u]}function o(){var e="",t=u;if("+"==c?i():"-"==c&&(e+=c,i()),!n(c))return a(t),null;for(;n(c);)e+=c,i();if("E"==c||"e"==c){if(e+=c,i(),("+"==c||"-"==c)&&(e+=c,i()),!r(c))return a(t),null;for(;r(c);)e+=c,i()}return e}function f(){var e=s[u+1];if("I"==c||"i"==c)return i(),"1";if(!("+"!=c&&"-"!=c||"I"!=e&&"i"!=e)){var t="+"==c?"1":"-1";return i(),i(),t}return null}var s,u,c;t.parse=function(n){if(s=n,u=-1,c="",!y(s))return null;i(),e();var r=o();if(r){if("I"==c||"i"==c)return i(),e(),c?null:new t(0,Number(r));e();var a=c;if("+"!=a&&"-"!=a)return e(),c?null:new t(Number(r),0);i(),e();var l=o();if(l){if("I"!=c&&"i"!=c)return null;i()}else if(l=f(),!l)return null;return"-"==a&&(l="-"==l[0]?"+"+l.substring(1):"-"+l),i(),e(),c?null:new t(Number(r),Number(l))}return(r=f())?(e(),c?null:new t(0,Number(r))):null}}(),t.prototype.clone=function(){return new t(this.re,this.im)},t.prototype.toString=function(){var e="",t=K.formatNumber(this.re),n=K.formatNumber(this.im);return e=0==this.im?t:0==this.re?1==this.im?"i":-1==this.im?"-i":n+"i":this.im>0?1==this.im?t+" + i":t+" + "+n+"i":-1==this.im?t+" - i":t+" - "+K.formatNumber(Math.abs(this.im))+"i"},Z.type.Matrix=n,n.prototype.get=function(e){var t;if(e instanceof n)t=e.isVector(),e=e.valueOf();else{if(!(e instanceof Array))throw new TypeError("Unsupported type of index "+Z.typeof(e));t=!e.some(function(e){return e.forEach})}if(e.length!=this._size.length)throw new RangeError("Dimension mismatch ("+e.length+" != "+this._size.length+")");if(t)switch(e.length){case 1:return i(this._data,e[0]);case 2:return i(i(this._data,e[0]),e[1]);default:return a(this._data,e)}else switch(e.length){case 1:return new n(o(this._data,e));case 2:return new n(f(this._data,e));default:return new n(s(this._data,e,0))}},n.prototype.set=function(e,t){var r;if(e instanceof n)r=e.isVector(),e=e.valueOf();else{if(!(e instanceof Array))throw new TypeError("Unsupported type of index "+Z.typeof(e));r=!e.some(function(e){return e.forEach})}if((t instanceof n||t instanceof w)&&(t=t.valueOf()),e.length<this._size.length)throw new RangeError("Dimension mismatch ("+e.length+" != "+this._size.length+")");if(r){if(0!=Z.size(t).length)throw new TypeError("Scalar value expected");switch(e.length){case 1:l(this._data,this._size,e,t);break;case 2:h(this._data,this._size,e,t);break;default:c(this._data,this._size,e,t)}}else{var i=this._size.concat();m(this._data,i,e,0,t),K.deepEqual(this._size,i)||(p(this._data),this.resize(i))}return this},n.prototype.resize=function(e,t){K.resize(this._data,e,t),this._size=Z.clone(e)},n.prototype.clone=function(){var e=new n;return e._data=Z.clone(this._data),e._size=Z.clone(this._size),e},n.prototype.size=function(){return this._size},n.prototype.map=function(e){var t=this,r=new n,i=[],a=function(n,r){return n instanceof Array?n.map(function(e,t){return i[r]=t+1,a(e,r+1)}):e(n,i,t)};return r._data=a(this._data,0),r._size=Z.clone(this._size),r},n.prototype.forEach=function(e){var t=this,n=[],r=function(i,a){i instanceof Array?i.forEach(function(e,t){n[a]=t+1,r(e,a+1)}):e(i,n,t)};r(this._data,0)},n.prototype.toScalar=function(){for(var e=this._data;e instanceof Array&&1==e.length;)e=e[0];return e instanceof Array?null:Z.clone(e)},n.prototype.isScalar=function(){return this._size.every(function(e){return 1>=e})},n.prototype.toVector=function(){var e=0,t=void 0,n=[];if(this._size.forEach(function(r,i){r>1&&(e++,t=i),n[i]=0}),0==e){var r=this.toScalar();return r?[r]:[]}if(1==e){var i=[],a=function(e){e instanceof Array?e.forEach(a):i.push(e)};return a(this._data),i}return null},n.prototype.isVector=function(){var e=0;return this._size.forEach(function(t){t>1&&e++}),1>=e},n.prototype.toArray=function(){return Z.clone(this._data)},n.prototype.valueOf=function(){return this._data},n.prototype.toString=function(){return Z.format(this._data)},Z.type.Range=w,w.parse=function(e){if(!y(e))return null;var t=e.split(":"),n=t.map(function(e){return Number(e)}),r=n.some(function(e){return isNaN(e)});if(r)return null;switch(n.length){case 2:return new w(n[0],1,n[1]);case 3:return new w(n[0],n[1],n[2]);default:return null}},w.prototype.clone=function(){return new w(this.start,this.step,this.end)},w.prototype.size=function(){var e=0,t=Number(this.start),n=Number(this.step),r=Number(this.end),i=r-t;return Z.sign(n)==Z.sign(i)?e=Math.floor(i/n)+1:0==i&&(e=1),isNaN(e)&&(e=0),[e]},w.prototype.forEach=function(e){var t=Number(this.start),n=Number(this.step),r=Number(this.end),i=0;if(n>0)for(;r>=t;)e(t,i,this),t+=n,i++;else if(0>n)for(;t>=r;)e(t,i,this),t+=n,i++},w.prototype.map=function(e){var t=[];return this.forEach(function(n,r,i){t[r]=e(n,r,i)}),t},w.prototype.toMatrix=function(){return new n(this.toArray())},w.prototype.toArray=function(){var e=[];return this.forEach(function(t,n){e[n]=t}),e},w.prototype.toVector=w.prototype.toArray,w.prototype.isVector=function(){return!0},w.prototype.toScalar=function(){var e=this.toArray();return 1==e.length?e[0]:null},w.prototype.isScalar=function(){return 1==this.size()[0]},w.prototype.valueOf=function(){return this.toArray()},w.prototype.toString=function(){var e=Z.format(Number(this.start));return 1!=this.step&&(e+=":"+Z.format(Number(this.step))),e+=":"+Z.format(Number(this.end))},Z.type.Selector=function X(e){if(!(this instanceof X))throw new SyntaxError("Selector constructor must be called with the new operator");this.value=e instanceof Z.type.Selector?e.value:e||void 0},Z.type.Selector.prototype={done:function(){return this.value},get:function(){var e=this.value;if(!e)throw Error("Selector value is undefined");if(e.get)return new Z.type.Selector(e.get.apply(e,arguments));if(e instanceof Array)return e=new n(e),new Z.type.Selector(e.get.apply(e,arguments).valueOf());throw Error("Selector value has no method get")},set:function(){var e=this.value;if(!e)throw Error("Selector value is undefined");if(e.set)return new Z.type.Selector(e.set.apply(e,arguments));if(e instanceof Array)return e=new n(e),new Z.type.Selector(e.set.apply(e,arguments).valueOf());throw Error("Selector value has no method set")},valueOf:function(){return this.value},toString:function(){return Z.format(this.value)}},Z.type.Unit=E,function(){function e(){for(;" "==u||"	"==u;)r()}function t(e){return e>="0"&&"9">=e||"."==e}function n(e){return e>="0"&&"9">=e}function r(){s++,u=f[s]}function i(e){s=e,u=f[s]}function a(){var e="",a=s;if("+"==u?r():"-"==u&&(e+=u,r()),!t(u))return i(a),null;for(;t(u);)e+=u,r();if("E"==u||"e"==u){if(e+=u,r(),("+"==u||"-"==u)&&(e+=u,r()),!n(u))return i(a),null;for(;n(u);)e+=u,r()}return e}function o(){var t="";for(e();u&&" "!=u&&"	"!=u;)t+=u,r();return t||null}var f,s,u;E.parse=function(t){if(f=t,s=-1,u="",!y(f))return null;r(),e();var n,i=a();return i?(n=o(),r(),e(),u?null:i&&n?new E(Number(i),n):null):(n=o(),r(),e(),u?null:new E(null,n))}}(),E.prototype.clone=function(){var e=new E;for(var t in this)this.hasOwnProperty(t)&&(e[t]=this[t]);return e},E.prototype._normalize=function(e){return(e+this.unit.offset)*this.unit.value*this.prefix.value},E.prototype._unnormalize=function(e,t){return void 0==t?e/this.unit.value/this.prefix.value-this.unit.offset:e/this.unit.value/t-this.unit.offset},E.isPlainUnit=function(e){return null!=N(e)},E.prototype.hasBase=function(e){return void 0===this.unit.base?void 0===e:this.unit.base===e},E.prototype.equalBase=function(e){return this.unit.base===e.unit.base},E.prototype.equals=function(e){return this.equalBase(e)&&this.value==e.value},E.prototype.in=function(e){var t;if(y(e)){if(t=new E(null,e),!this.equalBase(t))throw Error("Units do not match");return t.value=this.value,t}if(e instanceof E){if(!this.equalBase(e))throw Error("Units do not match");if(null!=e.value)throw Error("Cannot convert to a unit with a value");if(null==e.unit)throw Error("Unit expected on the right hand side of function in");return t=e.clone(),t.value=this.value,t.fixPrefix=!0,t}throw Error("String or Unit expected as parameter")},E.prototype.toNumber=function(e){var t=this.in(e),n=this.fixPrefix?t._bestPrefix():t.prefix;return t._unnormalize(t.value,n.value)},E.prototype.toString=function(){var e,t;if(this.fixPrefix)e=this._unnormalize(this.value),t=null!=this.value?K.formatNumber(e)+" ":"",t+=this.prefix.name+this.unit.name;else{var n=this._bestPrefix();e=this._unnormalize(this.value,n.value),t=null!=this.value?K.formatNumber(e)+" ":"",t+=n.name+this.unit.name}return t},E.prototype._bestPrefix=function(){var e=Math.abs(this.value/this.unit.value),t=E.PREFIX_NONE,n=Math.abs(Math.log(e/t.value)/Math.LN10-1.2),r=this.unit.prefixes;for(var i in r)if(r.hasOwnProperty(i)){var a=r[i];if(a.scientific){var o=Math.abs(Math.log(e/a.value)/Math.LN10-1.2);n>o&&(t=a,n=o)}}return t},E.PREFIXES={NONE:{"":{name:"",value:1,scientific:!0}},SHORT:{"":{name:"",value:1,scientific:!0},da:{name:"da",value:10,scientific:!1},h:{name:"h",value:100,scientific:!1},k:{name:"k",value:1e3,scientific:!0},M:{name:"M",value:1e6,scientific:!0},G:{name:"G",value:1e9,scientific:!0},T:{name:"T",value:1e12,scientific:!0},P:{name:"P",value:1e15,scientific:!0},E:{name:"E",value:1e18,scientific:!0},Z:{name:"Z",value:1e21,scientific:!0},Y:{name:"Y",value:1e24,scientific:!0},d:{name:"d",value:.1,scientific:!1},c:{name:"c",value:.01,scientific:!1},m:{name:"m",value:.001,scientific:!0},u:{name:"u",value:1e-6,scientific:!0},n:{name:"n",value:1e-9,scientific:!0},p:{name:"p",value:1e-12,scientific:!0},f:{name:"f",value:1e-15,scientific:!0},a:{name:"a",value:1e-18,scientific:!0},z:{name:"z",value:1e-21,scientific:!0},y:{name:"y",value:1e-24,scientific:!0}},LONG:{"":{name:"",value:1,scientific:!0},deca:{name:"deca",value:10,scientific:!1},hecto:{name:"hecto",value:100,scientific:!1},kilo:{name:"kilo",value:1e3,scientific:!0},mega:{name:"mega",value:1e6,scientific:!0},giga:{name:"giga",value:1e9,scientific:!0},tera:{name:"tera",value:1e12,scientific:!0},peta:{name:"peta",value:1e15,scientific:!0},exa:{name:"exa",value:1e18,scientific:!0},zetta:{name:"zetta",value:1e21,scientific:!0},yotta:{name:"yotta",value:1e24,scientific:!0},deci:{name:"deci",value:.1,scientific:!1},centi:{name:"centi",value:.01,scientific:!1},milli:{name:"milli",value:.001,scientific:!0},micro:{name:"micro",value:1e-6,scientific:!0},nano:{name:"nano",value:1e-9,scientific:!0},pico:{name:"pico",value:1e-12,scientific:!0},femto:{name:"femto",value:1e-15,scientific:!0},atto:{name:"atto",value:1e-18,scientific:!0},zepto:{name:"zepto",value:1e-21,scientific:!0},yocto:{name:"yocto",value:1e-24,scientific:!0}},BINARY_SHORT:{"":{name:"",value:1,scientific:!0},k:{name:"k",value:1024,scientific:!0},M:{name:"M",value:Math.pow(1024,2),scientific:!0},G:{name:"G",value:Math.pow(1024,3),scientific:!0},T:{name:"T",value:Math.pow(1024,4),scientific:!0},P:{name:"P",value:Math.pow(1024,5),scientific:!0},E:{name:"E",value:Math.pow(1024,6),scientific:!0},Z:{name:"Z",value:Math.pow(1024,7),scientific:!0},Y:{name:"Y",value:Math.pow(1024,8),scientific:!0},Ki:{name:"Ki",value:1024,scientific:!0},Mi:{name:"Mi",value:Math.pow(1024,2),scientific:!0},Gi:{name:"Gi",value:Math.pow(1024,3),scientific:!0},Ti:{name:"Ti",value:Math.pow(1024,4),scientific:!0},Pi:{name:"Pi",value:Math.pow(1024,5),scientific:!0},Ei:{name:"Ei",value:Math.pow(1024,6),scientific:!0},Zi:{name:"Zi",value:Math.pow(1024,7),scientific:!0},Yi:{name:"Yi",value:Math.pow(1024,8),scientific:!0}},BINARY_LONG:{"":{name:"",value:1,scientific:!0},kilo:{name:"kilo",value:1024,scientific:!0},mega:{name:"mega",value:Math.pow(1024,2),scientific:!0},giga:{name:"giga",value:Math.pow(1024,3),scientific:!0},tera:{name:"tera",value:Math.pow(1024,4),scientific:!0},peta:{name:"peta",value:Math.pow(1024,5),scientific:!0},exa:{name:"exa",value:Math.pow(1024,6),scientific:!0},zetta:{name:"zetta",value:Math.pow(1024,7),scientific:!0},yotta:{name:"yotta",value:Math.pow(1024,8),scientific:!0},kibi:{name:"kibi",value:1024,scientific:!0},mebi:{name:"mebi",value:Math.pow(1024,2),scientific:!0},gibi:{name:"gibi",value:Math.pow(1024,3),scientific:!0},tebi:{name:"tebi",value:Math.pow(1024,4),scientific:!0},pebi:{name:"pebi",value:Math.pow(1024,5),scientific:!0},exi:{name:"exi",value:Math.pow(1024,6),scientific:!0},zebi:{name:"zebi",value:Math.pow(1024,7),scientific:!0},yobi:{name:"yobi",value:Math.pow(1024,8),scientific:!0}}},E.PREFIX_NONE={name:"",value:1,scientific:!0},E.BASE_UNITS={NONE:{},LENGTH:{},MASS:{},TIME:{},CURRENT:{},TEMPERATURE:{},LUMINOUS_INTENSITY:{},AMOUNT_OF_SUBSTANCE:{},FORCE:{},SURFACE:{},VOLUME:{},ANGLE:{},BIT:{}};var Q=E.BASE_UNITS,J=E.PREFIXES;E.BASE_UNIT_NONE={},E.UNIT_NONE={name:"",base:E.BASE_UNIT_NONE,value:1,offset:0},E.UNITS=[{name:"meter",base:Q.LENGTH,prefixes:J.LONG,value:1,offset:0},{name:"inch",base:Q.LENGTH,prefixes:J.NONE,value:.0254,offset:0},{name:"foot",base:Q.LENGTH,prefixes:J.NONE,value:.3048,offset:0},{name:"yard",base:Q.LENGTH,prefixes:J.NONE,value:.9144,offset:0},{name:"mile",base:Q.LENGTH,prefixes:J.NONE,value:1609.344,offset:0},{name:"link",base:Q.LENGTH,prefixes:J.NONE,value:.201168,offset:0},{name:"rod",base:Q.LENGTH,prefixes:J.NONE,value:5.02921,offset:0},{name:"chain",base:Q.LENGTH,prefixes:J.NONE,value:20.1168,offset:0},{name:"angstrom",base:Q.LENGTH,prefixes:J.NONE,value:1e-10,offset:0},{name:"m",base:Q.LENGTH,prefixes:J.SHORT,value:1,offset:0},{name:"ft",base:Q.LENGTH,prefixes:J.NONE,value:.3048,offset:0},{name:"yd",base:Q.LENGTH,prefixes:J.NONE,value:.9144,offset:0},{name:"mi",base:Q.LENGTH,prefixes:J.NONE,value:1609.344,offset:0},{name:"li",base:Q.LENGTH,prefixes:J.NONE,value:.201168,offset:0},{name:"rd",base:Q.LENGTH,prefixes:J.NONE,value:5.02921,offset:0},{name:"ch",base:Q.LENGTH,prefixes:J.NONE,value:20.1168,offset:0},{name:"mil",base:Q.LENGTH,prefixes:J.NONE,value:254e-7,offset:0},{name:"m2",base:Q.SURFACE,prefixes:J.SHORT,value:1,offset:0},{name:"sqin",base:Q.SURFACE,prefixes:J.NONE,value:64516e-8,offset:0},{name:"sqft",base:Q.SURFACE,prefixes:J.NONE,value:.09290304,offset:0},{name:"sqyd",base:Q.SURFACE,prefixes:J.NONE,value:.83612736,offset:0},{name:"sqmi",base:Q.SURFACE,prefixes:J.NONE,value:2589988.110336,offset:0},{name:"sqrd",base:Q.SURFACE,prefixes:J.NONE,value:25.29295,offset:0},{name:"sqch",base:Q.SURFACE,prefixes:J.NONE,value:404.6873,offset:0},{name:"sqmil",base:Q.SURFACE,prefixes:J.NONE,value:6.4516e-10,offset:0},{name:"m3",base:Q.VOLUME,prefixes:J.SHORT,value:1,offset:0},{name:"L",base:Q.VOLUME,prefixes:J.SHORT,value:.001,offset:0},{name:"litre",base:Q.VOLUME,prefixes:J.LONG,value:.001,offset:0},{name:"cuin",base:Q.VOLUME,prefixes:J.NONE,value:16387064e-12,offset:0},{name:"cuft",base:Q.VOLUME,prefixes:J.NONE,value:.028316846592,offset:0},{name:"cuyd",base:Q.VOLUME,prefixes:J.NONE,value:.764554857984,offset:0},{name:"teaspoon",base:Q.VOLUME,prefixes:J.NONE,value:5e-6,offset:0},{name:"tablespoon",base:Q.VOLUME,prefixes:J.NONE,value:15e-6,offset:0},{name:"minim",base:Q.VOLUME,prefixes:J.NONE,value:6.161152e-8,offset:0},{name:"fluiddram",base:Q.VOLUME,prefixes:J.NONE,value:36966911e-13,offset:0},{name:"fluidounce",base:Q.VOLUME,prefixes:J.NONE,value:2957353e-11,offset:0},{name:"gill",base:Q.VOLUME,prefixes:J.NONE,value:.0001182941,offset:0},{name:"cup",base:Q.VOLUME,prefixes:J.NONE,value:.0002365882,offset:0},{name:"pint",base:Q.VOLUME,prefixes:J.NONE,value:.0004731765,offset:0},{name:"quart",base:Q.VOLUME,prefixes:J.NONE,value:.0009463529,offset:0},{name:"gallon",base:Q.VOLUME,prefixes:J.NONE,value:.003785412,offset:0},{name:"beerbarrel",base:Q.VOLUME,prefixes:J.NONE,value:.1173478,offset:0},{name:"oilbarrel",base:Q.VOLUME,prefixes:J.NONE,value:.1589873,offset:0},{name:"hogshead",base:Q.VOLUME,prefixes:J.NONE,value:.238481,offset:0},{name:"fldr",base:Q.VOLUME,prefixes:J.NONE,value:36966911e-13,offset:0},{name:"floz",base:Q.VOLUME,prefixes:J.NONE,value:2957353e-11,offset:0},{name:"gi",base:Q.VOLUME,prefixes:J.NONE,value:.0001182941,offset:0},{name:"cp",base:Q.VOLUME,prefixes:J.NONE,value:.0002365882,offset:0},{name:"pt",base:Q.VOLUME,prefixes:J.NONE,value:.0004731765,offset:0},{name:"qt",base:Q.VOLUME,prefixes:J.NONE,value:.0009463529,offset:0},{name:"gal",base:Q.VOLUME,prefixes:J.NONE,value:.003785412,offset:0},{name:"bbl",base:Q.VOLUME,prefixes:J.NONE,value:.1173478,offset:0},{name:"obl",base:Q.VOLUME,prefixes:J.NONE,value:.1589873,offset:0},{name:"g",base:Q.MASS,prefixes:J.SHORT,value:.001,offset:0},{name:"gram",base:Q.MASS,prefixes:J.LONG,value:.001,offset:0},{name:"ton",base:Q.MASS,prefixes:J.SHORT,value:907.18474,offset:0},{name:"tonne",base:Q.MASS,prefixes:J.SHORT,value:1e3,offset:0},{name:"grain",base:Q.MASS,prefixes:J.NONE,value:6479891e-11,offset:0},{name:"dram",base:Q.MASS,prefixes:J.NONE,value:.0017718451953125,offset:0},{name:"ounce",base:Q.MASS,prefixes:J.NONE,value:.028349523125,offset:0},{name:"poundmass",base:Q.MASS,prefixes:J.NONE,value:.45359237,offset:0},{name:"hundredweight",base:Q.MASS,prefixes:J.NONE,value:45.359237,offset:0},{name:"stick",base:Q.MASS,prefixes:J.NONE,value:.115,offset:0},{name:"gr",base:Q.MASS,prefixes:J.NONE,value:6479891e-11,offset:0},{name:"dr",base:Q.MASS,prefixes:J.NONE,value:.0017718451953125,offset:0},{name:"oz",base:Q.MASS,prefixes:J.NONE,value:.028349523125,offset:0},{name:"lbm",base:Q.MASS,prefixes:J.NONE,value:.45359237,offset:0},{name:"cwt",base:Q.MASS,prefixes:J.NONE,value:45.359237,offset:0},{name:"s",base:Q.TIME,prefixes:J.SHORT,value:1,offset:0},{name:"min",base:Q.TIME,prefixes:J.NONE,value:60,offset:0},{name:"h",base:Q.TIME,prefixes:J.NONE,value:3600,offset:0},{name:"seconds",base:Q.TIME,prefixes:J.LONG,value:1,offset:0},{name:"second",base:Q.TIME,prefixes:J.LONG,value:1,offset:0},{name:"sec",base:Q.TIME,prefixes:J.LONG,value:1,offset:0},{name:"minutes",base:Q.TIME,prefixes:J.NONE,value:60,offset:0},{name:"minute",base:Q.TIME,prefixes:J.NONE,value:60,offset:0},{name:"hours",base:Q.TIME,prefixes:J.NONE,value:3600,offset:0},{name:"hour",base:Q.TIME,prefixes:J.NONE,value:3600,offset:0},{name:"day",base:Q.TIME,prefixes:J.NONE,value:86400,offset:0},{name:"days",base:Q.TIME,prefixes:J.NONE,value:86400,offset:0},{name:"rad",base:Q.ANGLE,prefixes:J.NONE,value:1,offset:0},{name:"deg",base:Q.ANGLE,prefixes:J.NONE,value:.017453292519943295,offset:0},{name:"grad",base:Q.ANGLE,prefixes:J.NONE,value:.015707963267948967,offset:0},{name:"cycle",base:Q.ANGLE,prefixes:J.NONE,value:6.283185307179586,offset:0},{name:"A",base:Q.CURRENT,prefixes:J.SHORT,value:1,offset:0},{name:"ampere",base:Q.CURRENT,prefixes:J.LONG,value:1,offset:0},{name:"K",base:Q.TEMPERATURE,prefixes:J.NONE,value:1,offset:0},{name:"degC",base:Q.TEMPERATURE,prefixes:J.NONE,value:1,offset:273.15},{name:"degF",base:Q.TEMPERATURE,prefixes:J.NONE,value:1/1.8,offset:459.67},{name:"degR",base:Q.TEMPERATURE,prefixes:J.NONE,value:1/1.8,offset:0},{name:"kelvin",base:Q.TEMPERATURE,prefixes:J.NONE,value:1,offset:0},{name:"celsius",base:Q.TEMPERATURE,prefixes:J.NONE,value:1,offset:273.15},{name:"fahrenheit",base:Q.TEMPERATURE,prefixes:J.NONE,value:1/1.8,offset:459.67},{name:"rankine",base:Q.TEMPERATURE,prefixes:J.NONE,value:1/1.8,offset:0},{name:"mol",base:Q.AMOUNT_OF_SUBSTANCE,prefixes:J.NONE,value:1,offset:0},{name:"mole",base:Q.AMOUNT_OF_SUBSTANCE,prefixes:J.NONE,value:1,offset:0},{name:"cd",base:Q.LUMINOUS_INTENSITY,prefixes:J.NONE,value:1,offset:0},{name:"candela",base:Q.LUMINOUS_INTENSITY,prefixes:J.NONE,value:1,offset:0},{name:"N",base:Q.FORCE,prefixes:J.SHORT,value:1,offset:0},{name:"newton",base:Q.FORCE,prefixes:J.LONG,value:1,offset:0},{name:"lbf",base:Q.FORCE,prefixes:J.NONE,value:4.4482216152605,offset:0},{name:"poundforce",base:Q.FORCE,prefixes:J.NONE,value:4.4482216152605,offset:0},{name:"b",base:Q.BIT,prefixes:J.BINARY_SHORT,value:1,offset:0},{name:"bits",base:Q.BIT,prefixes:J.BINARY_LONG,value:1,offset:0},{name:"B",base:Q.BIT,prefixes:J.BINARY_SHORT,value:8,offset:0},{name:"bytes",base:Q.BIT,prefixes:J.BINARY_LONG,value:8,offset:0}],Z.E=Math.E,Z.LN2=Math.LN2,Z.LN10=Math.LN10,Z.LOG2E=Math.LOG2E,Z.LOG10E=Math.LOG10E,Z.PI=Math.PI,Z.SQRT1_2=Math.SQRT1_2,Z.SQRT2=Math.SQRT2,Z.I=new t(0,1),Z.pi=Z.PI,Z.e=Z.E,Z.i=Z.I,Z.expr.node.Node=b,b.prototype.eval=function(){throw Error("Cannot evaluate a Node interface")
},b.prototype.toString=function(){return""},M.prototype=new b,Z.expr.node.Symbol=M,M.prototype.hasParams=function(){return void 0!=this.params&&this.params.length>0},M.prototype.eval=function(){var e=this.fn;if(void 0===e)throw Error("Undefined symbol "+this.name);var t=this.params.map(function(e){return e.eval()});return e.apply(this,t)},M.prototype.toString=function(){if(this.name&&!this.params)return this.name;var e=this.name;return this.params&&this.params.length&&(e+="("+this.params.join(", ")+")"),e},A.prototype=new b,Z.expr.node.Constant=A,A.prototype.eval=function(){return this.value},A.prototype.toString=function(){return this.value?Z.format(this.value):""},S.prototype=new b,Z.expr.node.MatrixNode=S,function(){function e(t){return t.map(function(t){return t instanceof Array?e(t):t.eval()})}function t(e){for(var t=[],r=e.length,i=0;r>i;i++){for(var a=e[i],o=a.length,f=null,s=null,u=0;o>u;u++){var c,l=Z.clone(a[u]);if(l instanceof n){if(c=l.size(),l=l.valueOf(),1==c.length)l=[l],c=[1,c[0]];else if(c.length>2)throw Error("Cannot merge a multi dimensional matrix")}else l instanceof w?(l=[l.valueOf()],c=[1,l[0].length]):(c=[1,1],l=[[l]]);if(null==f)f=l,s=c[0];else{if(c[0]!=s)throw Error("Dimension mismatch ("+c[0]+" != "+s+")");for(var h=0;s>h;h++)f[h]=f[h].concat(l[h])}}t=t.concat(f)}return t}function r(e){return e.some(function(e){return e instanceof n||e instanceof w?!0:e instanceof Array?r(e):!1})}S.prototype.eval=function(){var i=e(this.nodes);return r(i)&&(i=t(i)),new n(i)},S.prototype.toString=function(){return K.formatArray(this.nodes)}}(),T.prototype=new b,Z.expr.node.Block=T,T.prototype.add=function(e,t){var n=this.params.length;this.params[n]=e,this.visible[n]=void 0!=t?t:!0},T.prototype.eval=function(){for(var e=[],t=0,n=this.params.length;n>t;t++){var r=this.params[t].eval();this.visible[t]&&e.push(r)}return e},T.prototype.toString=function(){for(var e=[],t=0,n=this.params.length;n>t;t++)this.visible[t]&&e.push("\n  "+(""+this.params[t]));return"["+e.join(",")+"\n]"},U.prototype=new b,Z.expr.node.Assignment=U,U.prototype.eval=function(){if(void 0===this.expr)throw Error("Undefined symbol "+this.name);var e,t=this.params;if(t&&t.length){var n=[];this.params.forEach(function(e){n.push(e.eval())});var r=this.expr.eval();if(void 0==this.result.value)throw Error("Undefined symbol "+this.name);var i=this.result();if(!i.set)throw new TypeError("Cannot apply a subset to object of type "+Z.typeof(i));e=i.set(n,r),this.result.value=e}else e=this.expr.eval(),this.result.value=e;return e},U.prototype.toString=function(){var e="";return e+=this.name,this.params&&this.params.length&&(e+="("+this.params.join(", ")+")"),e+=" = ",e+=""+this.expr},L.prototype=new b,Z.expr.node.Arguments=L,L.prototype.eval=function(){var e=this.object;if(void 0==e)throw Error("Node undefined");for(var t=e.eval(),n=this.params,r=[],i=0,a=n.length;a>i;i++)r[i]=n[i].eval();if(!t.get)throw new TypeError("Cannot apply arguments to object of type "+Z.typeof(t));return t.get(r)},L.prototype.toString=function(){var e=this.object?""+this.object:"";return this.params&&(e+="("+this.params.join(", ")+")"),e},R.prototype=new b,Z.expr.node.FunctionAssignment=R,R.prototype.createFunction=function(e,t,n,r){var i=function(){var t=n?n.length:0,i=arguments?arguments.length:0;if(t!=i)throw x(e,i,t);if(t>0)for(var a=0;t>a;a++)n[a].value=arguments[a];return r.eval()};return i.toString=function(){return e+"("+t.join(", ")+")"},i},R.prototype.eval=function(){for(var e=this.variables,t=this.values,n=0,r=e.length;r>n;n++)e[n].value=t[n];return this.result.value=this.def,this.def},R.prototype.toString=function(){return""+this.def},Z.expr.Scope=function(e,t){this.readonly=!1,t&&void 0!=t.readonly&&(this.readonly=t.readonly),this.parentScope=e,this.nestedScopes=void 0,this.symbols={},this.defs={},this.updates={},this.links={}},Z.expr.Scope.prototype={createNestedScope:function(){if(this.readonly)throw Error("Cannot create nested scope: Scope is read-only");var e=new Z.expr.Scope(this);return this.nestedScopes||(this.nestedScopes=[]),this.nestedScopes.push(e),e},clear:function(){if(this.readonly)throw Error("Cannot clear scope: Scope is read-only");if(this.symbols={},this.defs={},this.links={},this.updates={},this.nestedScopes)for(var e=this.nestedScopes,t=0,n=e.length;n>t;t++)e[t].clear()},createSymbol:function(e){var t=this.symbols[e];if(!t){var n=this.findDef(e);t=this.newSymbol(e,n),this.symbols[e]=t}return t},newSymbol:function(e,t){var r=this,i=function(){var t,a;if(!i.value&&(i.value=r.findDef(e),!i.value))throw Error("Undefined symbol "+e);if("function"==typeof i.value)return i.value.apply(null,arguments);if(i.value instanceof n||i.value instanceof w||i.value instanceof Array){if(arguments.length){var o=i.value instanceof Array?new n(i.value):i.value;for(t=[],a=0;arguments.length>a;a++)t[a]=arguments[a];return o.get(t)}return i.value}return i.value};return i.value=t,i.toString=function(){return i.value?""+i.value:""},i},createLink:function(e){var t=this.links[e];return t||(t=this.createSymbol(e),this.links[e]=t),t},createDef:function(e,t){if(this.readonly)throw Error("Cannot create variable: Scope is read-only");var n=this.defs[e];return n||(n=this.createSymbol(e),this.defs[e]=n),n&&void 0!=t&&(n.value=t),n},createUpdate:function(e){if(this.readonly)throw Error("Cannot update variable: Scope is read-only");var t=this.updates[e];return t||(t=this.createLink(e),this.updates[e]=t),t},createConstant:function(e,t){var n=this.newSymbol(e,t);return this.symbols[e]=n,this.defs[e]=n,n},findDef:function(e){var t;if(t=this.defs[e])return t;if(t=this.updates[e])return t;if(this.parentScope)return this.parentScope.findDef(e);var n=Z[e];if(n)return this.createConstant(e,n);if(E.isPlainUnit(e)){var r=new E(null,e);return this.createConstant(e,r)}return void 0},removeLink:function(e){delete this.links[e]},removeDef:function(e){delete this.defs[e]},removeUpdate:function(e){delete this.updates[e]},init:function(){var e=this.symbols,t=this.parentScope;for(var n in e)if(e.hasOwnProperty(n)){var r=e[n];r.value=t?t.findDef(n):void 0}this.nestedScopes&&this.nestedScopes.forEach(function(e){e.init()})},hasLink:function(e){if(this.links[e])return!0;if(this.nestedScopes)for(var t=this.nestedScopes,n=0,r=t.length;r>n;n++)if(t[n].hasLink(e))return!0;return!1},hasDef:function(e){return void 0!=this.defs[e]},hasUpdate:function(e){return void 0!=this.updates[e]},getUndefinedSymbols:function(){var e=this.symbols,t=[];for(var n in e)if(e.hasOwnProperty(n)){var r=e[n];void 0==r.value&&t.push(r)}return this.nestedScopes&&this.nestedScopes.forEach(function(e){t=t.concat(e.getUndefinedSymbols())}),t}},function(){function e(){W++,K=Y.charAt(W)}function n(){W=0,K=Y.charAt(0)}function r(){for(Q=H.NULL,X="";" "==K||"	"==K;)e();if("#"==K)for(;"\n"!=K&&""!=K;)e();if(""==K)return Q=H.DELIMITER,void 0;if("-"==K||","==K||"("==K||")"==K||"["==K||"]"==K||'"'==K||"\n"==K||";"==K||":"==K||"!"==K||"'"==K)return Q=H.DELIMITER,X+=K,e(),void 0;if(i(K))for(Q=H.DELIMITER;i(K);)X+=K,e();else if(o(K)){for(Q=H.NUMBER;o(K);)X+=K,e();if("E"==K||"e"==K)for(X+=K,e(),("+"==K||"-"==K)&&(X+=K,e()),f(K)||(Q=H.UNKNOWN);f(K);)X+=K,e()}else{if(!a(K)){for(Q=H.UNKNOWN;""!=K;)X+=K,e();throw k('Syntax error in part "'+X+'"')}for(Q=H.SYMBOL;a(K)||f(K);)X+=K,e()}}function i(e){return"&"==e||"|"==e||"<"==e||">"==e||"="==e||"+"==e||"/"==e||"*"==e||"%"==e||"^"==e||","==e||";"==e||"\n"==e||"!"==e||"'"==e}function a(e){return e>="a"&&"z">=e||e>="A"&&"Z">=e||"_"==e}function o(e){return e>="0"&&"9">=e||"."==e}function f(e){return e>="0"&&"9">=e}function s(e){n(),r();var t;if(t=""==X?new A(void 0):u(e),""!=X)throw Q==H.DELIMITER?F("Unknown operator "+X):k('Unexpected part "'+X+'"');return t}function u(e){var t,n,i;for("\n"!=X&&";"!=X&&""!=X&&(t=c(e));"\n"==X||";"==X;)n||(n=new T,t&&(i=";"!=X,n.add(t,i))),r(),"\n"!=X&&";"!=X&&""!=X&&(t=c(e),i=";"!=X,n.add(t,i));return n?n:(t||(t=c(e)),t)}function c(e){var t=l(e);if(!(e.readonly||t instanceof U)){var n="ans",r=void 0,i=e.createDef(n);return new U(n,r,t,i)}return t}function l(e){if(Q==H.SYMBOL&&"function"==X){if(r(),Q!=H.SYMBOL)throw k("Function name expected");var t=X;if(r(),"("!=X)throw k("Opening parenthesis ( expected");for(var n=e.createNestedScope(),i=[],a=[];;){if(r(),Q!=H.SYMBOL)throw k("Variable name expected");var o=X,f=n.createDef(o);if(i.push(o),a.push(f),r(),","!=X){if(")"==X)break;throw k('Comma , or closing parenthesis ) expected"')}}if(r(),"="!=X)throw k("Equal sign = expected");r();var s=m(n),u=e.createDef(t);return new R(t,i,a,s,u)}return h(e)}function h(e){var t=!1;Q==H.SYMBOL&&(t=e.hasLink(X));var n=m(e);if("="==X){if(!(n instanceof M))throw k("Symbol expected at the left hand side of assignment operator =");var i=n.name,a=n.params;t||e.removeLink(i),r();var o=m(e),f=n.hasParams()?e.createUpdate(i):e.createDef(i);return new U(i,a,o,f)}return n}function m(e){var t=p(e);if(":"==X){for(var n=[t];":"==X;)r(),n.push(p(e));if(n.length>3)throw new TypeError("Invalid range");var i="range",a=Z.range;t=new M(i,a,n)}return t}function p(e){for(var t=v(e),n={"in":"in"};void 0!==n[X];){var i=X,a=Z[n[i]];r();var o=[t,v(e)];t=new M(i,a,o)}return t}function v(e){var t=g(e);return t}function g(e){for(var t=w(e),n={"==":"equal","!=":"unequal","<":"smaller",">":"larger","<=":"smallereq",">=":"largereq"};void 0!==n[X];){var i=X,a=Z[n[i]];r();var o=[t,w(e)];t=new M(i,a,o)}return t}function w(e){for(var t=d(e),n={"+":"add","-":"subtract"};void 0!==n[X];){var i=X,a=Z[n[i]];r();var o=[t,d(e)];t=new M(i,a,o)}return t}function d(e){for(var t=y(e),n={"*":"multiply","/":"divide","%":"mod",mod:"mod"};void 0!==n[X];){var i=X,a=Z[n[i]];r();var o=[t,y(e)];t=new M(i,a,o)}return t}function y(e){if("-"==X){var t=X,n=Z.unaryminus;r();var i=[N(e)];return new M(t,n,i)}return N(e)}function N(e){for(var t=[O(e)];"^"==X;)r(),t.push(O(e));for(var n=t.pop();t.length;){var i=t.pop(),a="^",o=Z.pow,f=[i,n];n=new M(a,o,f)}return n}function O(e){for(var t=x(e);"!"==X;){var n=X,i=Z.factorial;r();var a=[t];t=new M(n,i,a)}return t}function x(e){for(var t=b(e);"'"==X;){var n=X,i=Z.transpose;r();var a=[t];t=new M(n,i,a)}return t}function b(e){return L(e)}function L(e){if(Q==H.SYMBOL){var t=X;r();var n=e.createLink(t),i=_(e),a=new M(t,n,i);return a}return z(e)}function _(e){var t=[];if("("==X){if(r(),")"!=X)for(t.push(m(e));","==X;)r(),t.push(m(e));if(")"!=X)throw k("Parenthesis ) missing");r()}return t}function z(t){if('"'==X){for(var n="",i="";""!=K&&('"'!=K||"\\"==i);)n+=K,i=K,e();if(r(),'"'!=X)throw k('End of string " missing');r();var a=new A(n);return a}return q(t)}function q(e){if("["==X){var t;for(r();"\n"==X;)r();if("]"!=X){var n=[],i=0,a=0;for(n[0]=[m(e)];","==X||";"==X;){for(","==X?a++:(i++,a=0,n[i]=[]),r();"\n"==X;)r();for(n[i][a]=m(e);"\n"==X;)r()}var o=n.length,f=n.length>0?n[0].length:0;for(i=1;o>i;i++)if(n[i].length!=f)throw F("Number of columns must match ("+n[i].length+" != "+f+")");if("]"!=X)throw k("End of matrix ] missing");r(),t=new S(n)}else r(),t=new S([[]]);return t}return I(e)}function I(e){if(Q==H.NUMBER){var n;n="."==X?0:Number(X),r();var i;if(Q==H.SYMBOL){if("i"==X||"I"==X)return i=new t(0,n),r(),new A(i);if(E.isPlainUnit(X))return i=new E(n,X),r(),new A(i);throw V('Unknown unit "'+X+'"')}var a=new A(n);return a}return P(e)}function P(e){if("("==X){r();var t=m(e);if(")"!=X)throw k("Parenthesis ) expected");return r(),t}return C(e)}function C(){throw""==X?k("Unexpected end of expression"):k("Value expected")}function B(){return void 0}function G(){return W-X.length+1}function D(e){var t=B(),n=G();return void 0===t?void 0===n?e:e+" (char "+n+")":e+" (line "+t+", char "+n+")"}function k(e){return new SyntaxError(D(e))}function V(e){return new TypeError(D(e))}function F(e){return Error(D(e))}Z.expr.Parser=function j(e){if(this.constructor!=j)throw new SyntaxError("Parser constructor must be called with the new operator");this.scope=new Z.expr.Scope(null,e)},Z.expr.Parser.prototype.parse=function(e,t){return Y=e||"",t||(t=this.scope),s(t)},Z.expr.Parser.prototype.eval=function(e){var t=this.parse(e);return t.eval()},Z.expr.Parser.prototype.get=function(e){var t=this.scope.findDef(e);return t?t.value:void 0},Z.expr.Parser.prototype.set=function(e,t){this.scope.createDef(e,t)},Z.expr.Parser.prototype.clear=function(){this.scope.clear()};var H={NULL:0,DELIMITER:1,NUMBER:2,SYMBOL:3,UNKNOWN:4},Y="",W=0,K="",X="",Q=H.NULL}(),function(){function e(){this.idMax=-1,this.updateSeq=0,this.parser=new Z.expr.Parser,this.scope=new Z.expr.Scope,this.nodes={},this.firstNode=void 0,this.lastNode=void 0}Z.expr.Workspace=e,e.prototype.clear=function(){this.nodes={},this.firstNode=void 0,this.lastNode=void 0},e.prototype.append=function(t){var n=this._getNewId(),r=this.lastNode?this.lastNode.scope:this.scope,i=new Z.expr.Scope(r),a=new e.Node({id:n,expression:t,parser:this.parser,scope:i,nextNode:void 0,previousNode:this.lastNode});return this.nodes[n]=a,this.firstNode||(this.firstNode=a),this.lastNode&&(this.lastNode.nextNode=a),this.lastNode=a,this._update([n]),n},e.prototype.insertBefore=function(t,n){var r=this.nodes[n];if(!r)throw'Node with id "'+n+'" not found';var i=r.previousNode,a=this._getNewId(),o=i?i.scope:this.scope,f=new Z.expr.Scope(o),s=new e.Node({id:a,expression:t,parser:this.parser,scope:f,nextNode:r,previousNode:i});this.nodes[a]=s,i?i.nextNode=s:this.firstNode=s,r.previousNode=s,r.scope.parentScope=s.scope;var u=this.getDependencies(a);return-1==u.indexOf(a)&&u.unshift(a),this._update(u),a},e.prototype.insertAfter=function(e,t){var n=this.nodes[t];if(!n)throw'Node with id "'+t+'" not found';return n==this.lastNode?this.append(e):this.insertBefore(t+1,e)},e.prototype.remove=function(e){var t=this.nodes[e];if(!t)throw'Node with id "'+e+'" not found';var n=this.getDependencies(e),r=t.previousNode,i=t.nextNode;r?r.nextNode=i:this.firstNode=i,i?i.previousNode=r:this.lastNode=r;var a=r?r.scope:this.scope;i&&(i.scope.parentScope=a),delete this.nodes[e],this._update(n)},e.prototype.replace=function(t,n){var r=this.nodes[n];if(!r)throw'Node with id "'+n+'" not found';var i=[n];e._merge(i,this.getDependencies(n));var a=r.previousNode;r.nextNode,a?a.scope:this.scope,r.setExpr(t),e._merge(i,this.getDependencies(n)),this._update(i)},e.Node=function(e){this.id=e.id,this.parser=e.parser,this.scope=e.scope,this.nextNode=e.nextNode,this.previousNode=e.previousNode,this.updateSeq=0,this.result=void 0,this.setExpr(e.expression)},e.Node.prototype.setExpr=function(e){this.expression=e||"",this.scope.clear(),this._parse()},e.Node.prototype.getExpr=function(){return this.expression},e.Node.prototype.getResult=function(){return this.result},e.Node.prototype._parse=function(){try{this.fn=this.parser.parse(this.expression,this.scope)}catch(e){var t="Error: "+((e.message||e)+"");this.fn=new A(t)}},e.Node.prototype.eval=function(){try{this.scope.init(),this.result=this.fn.eval()}catch(e){this.scope.init(),this.result="Error: "+((e.message||e)+"")}return this.result},e._merge=function(e,t){for(var n=0,r=t.length;r>n;n++){var i=t[n];-1==e.indexOf(i)&&e.push(i)}},e.prototype.getDependencies=function(t){var n,r=[],i=this.nodes[t];if(i){var a=i.scope.defs,o=i.scope.updates,f=[];for(n in a)a.hasOwnProperty(n)&&f.push(n);for(n in o)o.hasOwnProperty(n)&&-1==f.indexOf(n)&&f.push(n);for(var s=i.nextNode;s&&f.length;){for(var u=s.scope,c=0;f.length>c;){if(n=f[c],(u.hasLink(n)||u.hasUpdate(n))&&-1==r.indexOf(s.id)){r.push(s.id);var l=this.getDependencies(s.id);e._merge(r,l)}u.hasDef(n)&&(f.splice(c,1),c--),c++}s=s.nextNode}}return r},e.prototype.getExpr=function(e){var t=this.nodes[e];if(!t)throw'Node with id "'+e+'" not found';return t.getExpr()},e.prototype.getResult=function(e){var t=this.nodes[e];if(!t)throw'Node with id "'+e+'" not found';return t.getResult()},e.prototype._update=function(e){this.updateSeq++;for(var t=this.updateSeq,n=this.nodes,r=0,i=e.length;i>r;r++){var a=e[r],o=n[a];o&&(o.eval(),o.updateSeq=t)}},e.prototype.getChanges=function(e){var t=[],n=this.firstNode;for(e=e||0;n;)n.updateSeq>e&&t.push(n.id),n=n.nextNode;return{ids:t,updateSeq:this.updateSeq}},e.prototype._getNewId=function(){return this.idMax++,this.idMax},e.prototype.toString=function(){return JSON.stringify(this.toJSON())},e.prototype.toJSON=function(){for(var e=[],t=this.firstNode;t;){var n={id:t.id,expression:t.expression,dependencies:this.getDependencies(t.id)};try{n.result=t.getResult()}catch(r){n.result="Error: "+((r.message||r)+"")}e.push(n),t=t.nextNode}return e}}(),Z.abs=function(e){if(1!=arguments.length)throw x("abs",arguments.length,1);if(v(e))return Math.abs(e);if(e instanceof t)return Math.sqrt(e.re*e.re+e.im*e.im);if(e instanceof Array||e instanceof n)return K.map(e,Z.abs);if(e.valueOf()!==e)return Z.abs(e.valueOf());throw O("abs",e)},Z.add=function(e,r){if(2!=arguments.length)throw x("add",arguments.length,2);if(v(e)){if(v(r))return e+r;if(r instanceof t)return new t(e+r.re,r.im)}else if(e instanceof t){if(v(r))return new t(e.re+r,e.im);if(r instanceof t)return new t(e.re+r.re,e.im+r.im)}else if(e instanceof E&&r instanceof E){if(!e.equalBase(r))throw Error("Units do not match");if(null==e.value)throw Error("Unit on left hand side of operator + has an undefined value");if(null==r.value)throw Error("Unit on right hand side of operator + has an undefined value");var i=e.clone();return i.value+=r.value,i.fixPrefix=!1,i}if(y(e)||y(r))return e+r;if(e instanceof Array||e instanceof n||r instanceof Array||r instanceof n)return K.map2(e,r,Z.add);if(e.valueOf()!==e||r.valueOf()!==r)return Z.add(e.valueOf(),r.valueOf());throw O("add",e,r)},Z.ceil=function(e){if(1!=arguments.length)throw x("ceil",arguments.length,1);if(v(e))return Math.ceil(e);if(e instanceof t)return new t(Math.ceil(e.re),Math.ceil(e.im));if(e instanceof Array||e instanceof n)return K.map(e,Z.ceil);if(e.valueOf()!==e)return Z.ceil(e.valueOf());throw O("ceil",e)},Z.cube=function(e){if(1!=arguments.length)throw x("cube",arguments.length,1);if(v(e))return e*e*e;if(e instanceof t)return Z.multiply(Z.multiply(e,e),e);if(e instanceof Array||e instanceof n)return K.map(e,Z.cube);if(e.valueOf()!==e)return Z.cube(e.valueOf());throw O("cube",e)},Z.divide=function(e,r){if(2!=arguments.length)throw x("divide",arguments.length,2);if(v(e)){if(v(r))return e/r;if(r instanceof t)return _(new t(e,0),r)}if(e instanceof t){if(v(r))return _(e,new t(r,0));if(r instanceof t)return _(e,r)}if(e instanceof E&&v(r)){var i=e.clone();return i.value/=r,i}if(e instanceof Array||e instanceof n)return r instanceof Array||r instanceof n?Z.multiply(e,Z.inv(r)):K.map2(e,r,Z.divide);if(r instanceof Array||r instanceof n)return Z.multiply(e,Z.inv(r));if(e.valueOf()!==e||r.valueOf()!==r)return Z.divide(e.valueOf(),r.valueOf());throw O("divide",e,r)},Z.equal=function $(e,r){if(2!=arguments.length)throw x("equal",arguments.length,2);if(v(e)){if(v(r))return e==r;if(r instanceof t)return e==r.re&&0==r.im}if(e instanceof t){if(v(r))return e.re==r&&0==e.im;if(r instanceof t)return e.re==r.re&&e.im==r.im}if(e instanceof E&&r instanceof E){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value==r.value}if(y(e)||y(r))return e==r;if(e instanceof Array||e instanceof n||r instanceof Array||r instanceof n)return K.map2(e,r,Z.equal);if(e.valueOf()!==e||r.valueOf()!==r)return $(e.valueOf(),r.valueOf());throw O("equal",e,r)},Z.exp=function(e){if(1!=arguments.length)throw x("exp",arguments.length,1);if(v(e))return Math.exp(e);if(e instanceof t){var r=Math.exp(e.re);return new t(r*Math.cos(e.im),r*Math.sin(e.im))}if(e instanceof Array||e instanceof n)return K.map(e,Z.exp);if(e.valueOf()!==e)return Z.exp(e.valueOf());throw O("exp",e)},Z.fix=function(e){if(1!=arguments.length)throw x("fix",arguments.length,1);if(v(e))return e>0?Math.floor(e):Math.ceil(e);if(e instanceof t)return new t(e.re>0?Math.floor(e.re):Math.ceil(e.re),e.im>0?Math.floor(e.im):Math.ceil(e.im));if(e instanceof Array||e instanceof n)return K.map(e,Z.fix);if(e.valueOf()!==e)return Z.fix(e.valueOf());throw O("fix",e)},Z.floor=function(e){if(1!=arguments.length)throw x("floor",arguments.length,1);if(v(e))return Math.floor(e);if(e instanceof t)return new t(Math.floor(e.re),Math.floor(e.im));if(e instanceof Array||e instanceof n)return K.map(e,Z.floor);if(e.valueOf()!==e)return Z.floor(e.valueOf());throw O("floor",e)},Z.gcd=function(){var e,t=arguments[0],r=arguments[1];if(2==arguments.length){if(v(t)&&v(r)){if(!g(t)||!g(r))throw Error("Parameters in function gcd must be integer numbers");for(;0!=r;)e=r,r=t%e,t=e;return Math.abs(t)}if(t instanceof Array||t instanceof n||r instanceof Array||r instanceof n)return K.map2(t,r,Z.gcd);if(t.valueOf()!==t||r.valueOf()!==r)return Z.gcd(t.valueOf(),r.valueOf());throw O("gcd",t,r)}if(arguments.length>2){for(var i=1;arguments.length>i;i++)t=Z.gcd(t,arguments[i]);return t}throw new SyntaxError("Function gcd expects two or more arguments")},Z.larger=function(e,r){if(2!=arguments.length)throw x("larger",arguments.length,2);if(v(e)){if(v(r))return e>r;if(r instanceof t)return e>Z.abs(r)}if(e instanceof t){if(v(r))return Z.abs(e)>r;if(r instanceof t)return Z.abs(e)>Z.abs(r)}if(e instanceof E&&r instanceof E){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value>r.value}if(y(e)||y(r))return e>r;if(e instanceof Array||e instanceof n||r instanceof Array||r instanceof n)return K.map2(e,r,Z.larger);if(e.valueOf()!==e||r.valueOf()!==r)return Z.larger(e.valueOf(),r.valueOf());throw O("larger",e,r)},Z.largereq=function(e,r){if(2!=arguments.length)throw x("largereq",arguments.length,2);if(v(e)){if(v(r))return e>=r;if(r instanceof t)return e>=Z.abs(r)}if(e instanceof t){if(v(r))return Z.abs(e)>=r;if(r instanceof t)return Z.abs(e)>=Z.abs(r)}if(e instanceof E&&r instanceof E){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value>=r.value}if(y(e)||y(r))return e>=r;if(e instanceof Array||e instanceof n||r instanceof Array||r instanceof n)return K.map2(e,r,Z.largereq);if(e.valueOf()!==e||r.valueOf()!==r)return Z.largereq(e.valueOf(),r.valueOf());throw O("largereq",e,r)},Z.lcm=function(){var e,t=arguments[0],r=arguments[1];if(2==arguments.length){if(v(t)&&v(r)){if(!g(t)||!g(r))throw Error("Parameters in function lcm must be integer numbers");for(var i=t*r;0!=r;)e=r,r=t%e,t=e;return Math.abs(i/t)}if(t instanceof Array||t instanceof n||r instanceof Array||r instanceof n)return K.map2(t,r,Z.lcm);if(t.valueOf()!==t||r.valueOf()!==r)return Z.lcm(t.valueOf(),r.valueOf());throw O("lcm",t,r)}if(arguments.length>2){for(var a=1;arguments.length>a;a++)t=Z.lcm(t,arguments[a]);return t}throw new SyntaxError("Function lcm expects two or more arguments")},Z.log=function(e,r){if(1!=arguments.length&&2!=arguments.length)throw x("log",arguments.length,1,2);if(void 0!==r)return Z.divide(Z.log(e),Z.log(r));if(v(e))return e>=0?Math.log(e):Z.log(new t(e,0));if(e instanceof t)return new t(Math.log(Math.sqrt(e.re*e.re+e.im*e.im)),Math.atan2(e.im,e.re));if(e instanceof Array||e instanceof n)return K.map(e,Z.log);if(e.valueOf()!==e||r.valueOf()!==r)return Z.log(e.valueOf(),r.valueOf());throw O("log",e,r)},Z.log10=function(e){if(1!=arguments.length)throw x("log10",arguments.length,1);if(v(e))return e>=0?Math.log(e)/Math.LN10:Z.log10(new t(e,0));if(e instanceof t)return new t(Math.log(Math.sqrt(e.re*e.re+e.im*e.im))/Math.LN10,Math.atan2(e.im,e.re)/Math.LN10);if(e instanceof Array||e instanceof n)return K.map(e,Z.log10);if(e.valueOf()!==e)return Z.log10(e.valueOf());throw O("log10",e)},Z.mod=function(e,r){if(2!=arguments.length)throw x("mod",arguments.length,2);if(v(e)){if(v(r))return e%r;if(r instanceof t&&0==r.im)return e%r.re}else if(e instanceof t&&0==e.im){if(v(r))return e.re%r;if(r instanceof t&&0==r.im)return e.re%r.re}if(e instanceof Array||e instanceof n||r instanceof Array||r instanceof n)return K.map2(e,r,Z.mod);if(e.valueOf()!==e||r.valueOf()!==r)return Z.mod(e.valueOf(),r.valueOf());throw O("mod",e,r)},Z.multiply=function(e,r){if(2!=arguments.length)throw x("multiply",arguments.length,2);if(v(e)){if(v(r))return e*r;if(r instanceof t)return z(new t(e,0),r);if(r instanceof E)return o=r.clone(),o.value*=e,o}else if(e instanceof t){if(v(r))return z(e,new t(r,0));if(r instanceof t)return z(e,r)}else if(e instanceof E){if(v(r))return o=e.clone(),o.value*=r,o}else{if(e instanceof Array){if(r instanceof Array){var i=K.size(e),a=K.size(r);if(2!=i.length)throw Error("Can only multiply a 2 dimensional matrix (A has "+i.length+" dimensions)");if(2!=a.length)throw Error("Can only multiply a 2 dimensional matrix (B has "+a.length+" dimensions)");if(i[1]!=a[0])throw new RangeError("Dimensions mismatch in multiplication. Columns of A must match rows of B (A is "+i[0]+"x"+i[1]+", B is "+a[0]+"x"+a[1]+", "+a[1]+" != "+a[0]+")");for(var o=[],f=i[0],s=a[1],u=i[1],c=Z.multiply,l=Z.add,h=0;f>h;h++){o[h]=[];for(var m=0;s>m;m++){for(var p=null,g=0;u>g;g++){var w=c(e[h][g],r[g][m]);p=null==p?w:l(p,w)}o[h][m]=p}}return o}return r instanceof n?new n(Z.multiply(e.valueOf(),r.valueOf())):K.map2(e,r,Z.multiply)}if(e instanceof n)return new n(Z.multiply(e.valueOf(),r.valueOf()))}if(r instanceof Array)return K.map2(e,r,Z.multiply);if(r instanceof n)return new n(Z.multiply(e.valueOf(),r.valueOf()));if(e.valueOf()!==e||r.valueOf()!==r)return Z.multiply(e.valueOf(),r.valueOf());throw O("multiply",e,r)},Z.pow=function(e,r){if(2!=arguments.length)throw x("pow",arguments.length,2);if(v(e)){if(v(r))return g(r)||e>=0?Math.pow(e,r):q(new t(e,0),new t(r,0));if(r instanceof t)return q(new t(e,0),r)}else if(e instanceof t){if(v(r))return q(e,new t(r,0));if(r instanceof t)return q(e,r)}else{if(e instanceof Array){if(!v(r)||!g(r)||0>r)throw new TypeError("For A^b, b must be a positive integer (value is "+r+")");var i=K.size(e);if(2!=i.length)throw Error("For A^b, A must be 2 dimensional (A has "+i.length+" dimensions)");if(i[0]!=i[1])throw Error("For A^b, A must be square (size is "+i[0]+"x"+i[1]+")");if(0==r)return Z.eye(i[0]);for(var a=e,o=1;r>o;o++)a=Z.multiply(e,a);return a}if(e instanceof n)return new n(Z.pow(e.valueOf(),r))}if(e.valueOf()!==e||r.valueOf()!==r)return Z.pow(e.valueOf(),r.valueOf());throw O("pow",e,r)},Z.round=function(e,r){if(1!=arguments.length&&2!=arguments.length)throw x("round",arguments.length,1,2);if(void 0==r){if(v(e))return Math.round(e);if(e instanceof t)return new t(Math.round(e.re),Math.round(e.im));if(e instanceof Array||e instanceof n)return K.map(e,Z.round);if(e.valueOf()!==e)return Z.round(e.valueOf());throw O("round",e)}if(!v(r))throw new TypeError("Number of decimals in function round must be an integer");if(r!==Math.round(r))throw new TypeError("Number of decimals in function round must be integer");if(0>r||r>9)throw Error("Number of decimals in function round must be in te range of 0-9");if(v(e))return I(e,r);if(e instanceof t)return new t(I(e.re,r),I(e.im,r));if(e instanceof Array||e instanceof n||r instanceof Array||r instanceof n)return K.map2(e,r,Z.round);if(e.valueOf()!==e||r.valueOf()!==r)return Z.round(e.valueOf(),r.valueOf());throw O("round",e,r)},Z.sign=function(e){if(1!=arguments.length)throw x("sign",arguments.length,1);if(v(e)){var r;return r=e>0?1:0>e?-1:0}if(e instanceof t){var i=Math.sqrt(e.re*e.re+e.im*e.im);return new t(e.re/i,e.im/i)}if(e instanceof Array||e instanceof n)return K.map(e,Z.sign);if(e.valueOf()!==e)return Z.sign(e.valueOf());throw O("sign",e)},Z.smaller=function(e,r){if(2!=arguments.length)throw x("smaller",arguments.length,2);if(v(e)){if(v(r))return r>e;if(r instanceof t)return Z.abs(r)>e}if(e instanceof t){if(v(r))return r>Z.abs(e);if(r instanceof t)return Z.abs(e)<Z.abs(r)}if(e instanceof E&&r instanceof E){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value<r.value}if(y(e)||y(r))return r>e;if(e instanceof Array||e instanceof n||r instanceof Array||r instanceof n)return K.map2(e,r,Z.smaller);if(e.valueOf()!==e||r.valueOf()!==r)return Z.smaller(e.valueOf(),r.valueOf());throw O("smaller",e,r)},Z.smallereq=function(e,r){if(2!=arguments.length)throw x("smallereq",arguments.length,2);if(v(e)){if(v(r))return r>=e;if(r instanceof t)return Z.abs(r)>=e}if(e instanceof t){if(v(r))return r>=Z.abs(e);if(r instanceof t)return Z.abs(e)<=Z.abs(r)}if(e instanceof E&&r instanceof E){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value<=r.value}if(y(e)||y(r))return r>=e;if(e instanceof Array||e instanceof n||r instanceof Array||r instanceof n)return K.map2(e,r,Z.smallereq);if(e.valueOf()!==e||r.valueOf()!==r)return Z.smallereq(e.valueOf(),r.valueOf());throw O("smallereq",e,r)},Z.sqrt=function(e){if(1!=arguments.length)throw x("sqrt",arguments.length,1);if(v(e))return e>=0?Math.sqrt(e):Z.sqrt(new t(e,0));if(e instanceof t){var r=Math.sqrt(e.re*e.re+e.im*e.im);return e.im>=0?new t(.5*Math.sqrt(2*(r+e.re)),.5*Math.sqrt(2*(r-e.re))):new t(.5*Math.sqrt(2*(r+e.re)),-.5*Math.sqrt(2*(r-e.re)))}if(e instanceof Array||e instanceof n)return K.map(e,Z.sqrt);if(e.valueOf()!==e)return Z.sqrt(e.valueOf());throw O("sqrt",e)},Z.square=function(e){if(1!=arguments.length)throw x("square",arguments.length,1);if(v(e))return e*e;if(e instanceof t)return Z.multiply(e,e);if(e instanceof Array||e instanceof n)return K.map(e,Z.square);if(e.valueOf()!==e)return Z.square(e.valueOf());throw O("square",e)},Z.subtract=function(e,r){if(2!=arguments.length)throw x("subtract",arguments.length,2);if(v(e)){if(v(r))return e-r;if(r instanceof t)return new t(e-r.re,r.im)}else if(e instanceof t){if(v(r))return new t(e.re-r,e.im);if(r instanceof t)return new t(e.re-r.re,e.im-r.im)}else if(e instanceof E&&r instanceof E){if(!e.equalBase(r))throw Error("Units do not match");if(null==e.value)throw Error("Unit on left hand side of operator - has an undefined value");if(null==r.value)throw Error("Unit on right hand side of operator - has an undefined value");var i=e.clone();return i.value-=r.value,i.fixPrefix=!1,i}if(e instanceof Array||e instanceof n||r instanceof Array||r instanceof n)return K.map2(e,r,Z.subtract);if(e.valueOf()!==e||r.valueOf()!==r)return Z.subtract(e.valueOf(),r.valueOf());throw O("subtract",e,r)},Z.unaryminus=function(e){if(1!=arguments.length)throw x("unaryminus",arguments.length,1);if(v(e))return-e;if(e instanceof t)return new t(-e.re,-e.im);if(e instanceof E){var r=e.clone();return r.value=-e.value,r}if(e instanceof Array||e instanceof n)return K.map(e,Z.unaryminus);if(e.valueOf()!==e)return Z.unaryminus(e.valueOf());throw O("unaryminus",e)},Z.unequal=function(e,r){if(2!=arguments.length)throw x("unequal",arguments.length,2);if(v(e)){if(v(r))return e==r;if(r instanceof t)return e==r.re&&0==r.im}if(e instanceof t){if(v(r))return e.re==r&&0==e.im;if(r instanceof t)return e.re==r.re&&e.im==r.im}if(e instanceof E&&r instanceof E){if(!e.equalBase(r))throw Error("Cannot compare units with different base");return e.value==r.value}if(y(e)||y(r))return e==r;if(e instanceof Array||e instanceof n||r instanceof Array||r instanceof n)return K.map2(e,r,Z.unequal);if(e.valueOf()!==e||r.valueOf()!==r)return Z.unequal(e.valueOf(),r.valueOf());throw O("unequal",e,r)},Z.xgcd=function et(e,t){if(2==arguments.length){if(v(e)&&v(t)){if(!g(e)||!g(t))throw Error("Parameters in function xgcd must be integer numbers");if(0==t)return[e,1,0];var n=et(t,e%t),r=n[0],i=n[1],a=n[2];return[r,a,i-a*Math.floor(e/t)]}throw O("xgcd",e,t)}throw new SyntaxError("Function xgcd expects two arguments")},Z.arg=function(e){if(1!=arguments.length)throw x("arg",arguments.length,1);return v(e)?Math.atan2(0,e):e instanceof t?Math.atan2(e.im,e.re):e instanceof Array||e instanceof n?K.map(e,Z.arg):e.valueOf()!==e?Z.arg(e.valueOf()):Z.atan2(0,e)},Z.conj=function(e){if(1!=arguments.length)throw x("conj",arguments.length,1);return v(e)?e:e instanceof t?new t(e.re,-e.im):e instanceof Array||e instanceof n?K.map(e,Z.conj):e.valueOf()!==e?Z.conj(e.valueOf()):tt(e)},Z.im=function(e){if(1!=arguments.length)throw x("im",arguments.length,1);return v(e)?0:e instanceof t?e.im:e instanceof Array||e instanceof n?K.map(e,Z.im):e.valueOf()!==e?Z.im(e.valueOf()):0
},Z.re=function(e){if(1!=arguments.length)throw x("re",arguments.length,1);return v(e)?e:e instanceof t?e.re:e instanceof Array||e instanceof n?K.map(e,Z.re):e.valueOf()!==e?Z.re(e.valueOf()):Z.clone(e)},Z.complex=function(){switch(arguments.length){case 0:return new t(0,0);case 1:var e=arguments[0];if(!y(e))throw new TypeError("Two numbers or a single string expected in function complex");var n=t.parse(e);if(n)return n;throw new SyntaxError('String "'+e+'" is no valid complex number');case 2:return new t(arguments[0],arguments[1]);default:throw x("complex",arguments.length,0,2)}},Z.matrix=function(e){if(arguments.length>1)throw x("matrix",arguments.length,0,1);return new n(e)},Z.parser=function(){return new Z.expr.Parser},Z.range=function(e){switch(arguments.length){case 1:if(!y(e))throw new TypeError("Two or three numbers or a single string expected in function range");var t=w.parse(e);if(t)return t;throw new SyntaxError('String "'+t+'" is no valid range');case 2:return new w(arguments[0],null,arguments[1]);case 3:return new w(arguments[0],arguments[1],arguments[2]);default:throw x("range",arguments.length,2,3)}},Z.unit=function(){switch(arguments.length){case 1:var e=arguments[0];if(!y(e))throw new TypeError("A string or a number and string expected in function unit");if(E.isPlainUnit(e))return new E(null,e);var t=E.parse(e);if(t)return t;throw new SyntaxError('String "'+e+'" is no valid unit');case 2:return new E(arguments[0],arguments[1]);default:throw x("unit",arguments.length,1,2)}},Z.workspace=function(){return new Z.expr.Workspace},Z.concat=function(){var e,t,r=arguments.length,i=-1,a=!1,o=[];for(e=0;r>e;e++){var f=arguments[e];if(f instanceof n&&(a=!0),e==r-1&&v(f)){if(t=i,i=f,!g(i)||1>i)throw new TypeError("Dimension number must be a positive integer (dim = "+i+")");if(e>0&&i>t)throw new RangeError("Dimension out of range ("+i+" > "+t+")")}else{if(!(f instanceof Array||f instanceof n))throw O("concat",f);var s=Z.clone(f.valueOf()),u=Z.size(f);if(o[e]=s,t=i,i=u.length,e>0&&i!=t)throw new RangeError("Dimension mismatch ("+t+" != "+i+")")}}if(0==o.length)throw new SyntaxError("At least one matrix expected");for(var c=o.shift();o.length;)c=P(c,o.shift(),i-1,0);return a?new n(c):c},Z.det=function(e){if(1!=arguments.length)throw x("det",arguments.length,1);var t=Z.size(e);switch(t.length){case 0:return Z.clone(e);case 1:if(1==t[0])return Z.clone(e.valueOf()[0]);throw new RangeError("Matrix must be square (size: "+Z.format(t)+")");case 2:var n=t[0],r=t[1];if(n==r)return C(e.valueOf(),n,r);throw new RangeError("Matrix must be square (size: "+Z.format(t)+")");default:throw new RangeError("Matrix must be two dimensional (size: "+Z.format(t)+")")}},Z.diag=function(e,t){var r,i,a,o;if(1!=arguments.length&&2!=arguments.length)throw x("diag",arguments.length,1,2);if(t){if(!v(t)||!g(t))throw new TypeError("Second parameter in function diag must be an integer")}else t=0;var f=t>0?t:0,s=0>t?-t:0;e instanceof n||e instanceof w||(e=new n(e));var u;switch(e.isVector()?(e=e.toVector(),u=[e.length]):u=e.size(),u.length){case 1:i=e.valueOf();var c=new n;for(c.resize([i.length+s,i.length+f]),r=c.valueOf(),o=i.length,a=0;o>a;a++)r[a+s][a+f]=Z.clone(i[a]);return c;case 2:for(i=[],r=e.valueOf(),o=Math.min(u[0]-s,u[1]-f),a=0;o>a;a++)i[a]=Z.clone(r[a+s][a+f]);return new n(i);default:throw new RangeError("Matrix for function diag must be 2 dimensional")}},Z.eye=function(){var e=K.argsToArray(arguments);if(0==e.length)e=[1,1];else if(1==e.length)e[1]=e[0];else if(e.length>2)throw x("eye",e.length,0,2);var t=e[0],r=e[1];if(!v(t)||!g(t)||1>t)throw Error("Parameters in function eye must be positive integers");if(r&&(!v(r)||!g(r)||1>r))throw Error("Parameters in function eye must be positive integers");var i=new n;i.resize(e);for(var a=Z.min(e),o=i.valueOf(),f=0;a>f;f++)o[f][f]=1;return i},Z.inv=function(e){if(1!=arguments.length)throw x("inv",arguments.length,1);var t=Z.size(e);switch(t.length){case 0:return Z.divide(1,e);case 1:if(1==t[0])return e instanceof n?new n([Z.divide(1,e.valueOf()[0])]):[Z.divide(1,e[0])];throw new RangeError("Matrix must be square (size: "+Z.format(t)+")");case 2:var r=t[0],i=t[1];if(r==i)return e instanceof n?new n(G(e.valueOf(),r,i)):G(e,r,i);throw new RangeError("Matrix must be square (size: "+Z.format(t)+")");default:throw new RangeError("Matrix must be two dimensional (size: "+Z.format(t)+")")}},Z.ones=function(){var e=K.argsToArray(arguments);0==e.length?e=[1,1]:1==e.length&&(e[1]=e[0]);var t=new n,r=1;return t.resize(e,r),t},Z.size=function(e){if(1!=arguments.length)throw x("size",arguments.length,1);if(v(e)||e instanceof t||e instanceof E||null==e)return[];if(y(e))return[e.length];if(e instanceof Array)return K.size(e);if(e instanceof n)return e.size();if(e.valueOf()!==e)return Z.size(e.valueOf());throw O("size",e)},Z.squeeze=function(e){if(1!=arguments.length)throw x("squeeze",arguments.length,1);return e instanceof Array?D(Z.clone(e)):e instanceof n?D(e.toArray()):e.valueOf()instanceof Array?D(Z.clone(e.valueOf())):Z.clone(e)},Z.transpose=function(e){if(1!=arguments.length)throw x("transpose",arguments.length,1);var t=Z.size(e);switch(t.length){case 0:return Z.clone(e);case 1:return Z.clone(e);case 2:for(var r,i=t[1],a=t[0],o=e instanceof n,f=e.valueOf(),s=[],u=Z.clone,c=0;i>c;c++){r=s[c]=[];for(var l=0;a>l;l++)r[l]=u(f[l][c])}return 0==a&&(s[0]=[]),o?new n(s):s;default:throw new RangeError("Matrix must be two dimensional (size: "+Z.format(t)+")")}},Z.zeros=function(){var e=K.argsToArray(arguments);0==e.length?e=[1,1]:1==e.length&&(e[1]=e[0]);var t=new n;return t.resize(e),t},Z.factorial=function(e){if(1!=arguments.length)throw x("factorial",arguments.length,1);if(v(e)){if(!g(e)||0>e)throw new TypeError("Positive integer value expected in function factorial");var t=e,r=t;for(t--;t>1;)r*=t,t--;return 0==r&&(r=1),r}if(e instanceof Array||e instanceof n)return K.map(e,Z.factorial);if(e.valueOf()!==e)return Z.factorial(e.valueOf());throw O("factorial",e)},Z.random=function(){if(0!=arguments.length)throw x("random",arguments.length,0);return Math.random()},Z.max=function(e){if(0==arguments.length)throw Error("Function max requires one or more parameters (0 provided)");if(e instanceof Array||e instanceof n||e instanceof w){if(arguments.length>1)throw Error("Wrong number of parameters (1 matrix or multiple scalars expected)");var t=Z.size(e);if(1==t.length){if(0==e.length)throw Error("Cannot calculate max of an empty vector");return k(e.valueOf())}if(2==t.length){if(0==t[0]||0==t[1])throw Error("Cannot calculate max of an empty matrix");if(e instanceof Array)return V(e,t[0],t[1]);if(e instanceof n||e instanceof w)return new n(V(e.valueOf(),t[0],t[1]));throw O("max",e)}throw new RangeError("Cannot calculate max for multi dimensional matrix")}return k(arguments)},Z.min=function(e){if(0==arguments.length)throw Error("Function min requires one or more parameters (0 provided)");if(e instanceof Array||e instanceof n||e instanceof w){if(arguments.length>1)throw Error("Wrong number of parameters (1 matrix or multiple scalars expected)");var t=Z.size(e);if(1==t.length){if(0==e.length)throw Error("Cannot calculate min of an empty vector");return F(e.valueOf())}if(2==t.length){if(0==t[0]||0==t[1])throw Error("Cannot calculate min of an empty matrix");if(e instanceof Array)return j(e,t[0],t[1]);if(e instanceof n||e instanceof w)return new n(j(e.valueOf(),t[0],t[1]));throw O("min",e)}throw new RangeError("Cannot calculate min for multi dimensional matrix")}return F(arguments)},Z.acos=function(e){if(1!=arguments.length)throw x("acos",arguments.length,1);if(v(e))return e>=-1&&1>=e?Math.acos(e):Z.acos(new t(e,0));if(e instanceof t){var r=new t(e.im*e.im-e.re*e.re+1,-2*e.re*e.im),i=Z.sqrt(r),a=new t(i.re-e.im,i.im+e.re),o=Z.log(a);return new t(1.5707963267948966-o.im,o.re)}if(e instanceof Array||e instanceof n)return K.map(e,Z.acos);if(e.valueOf()!==e)return Z.acos(e.valueOf());throw O("acos",e)},Z.asin=function(e){if(1!=arguments.length)throw x("asin",arguments.length,1);if(v(e))return e>=-1&&1>=e?Math.asin(e):Z.asin(new t(e,0));if(e instanceof t){var r=e.re,i=e.im,a=new t(i*i-r*r+1,-2*r*i),o=Z.sqrt(a),f=new t(o.re-i,o.im+r),s=Z.log(f);return new t(s.im,-s.re)}if(e instanceof Array||e instanceof n)return K.map(e,Z.asin);if(e.valueOf()!==e)return Z.asin(e.valueOf());throw O("asin",e)},Z.atan=function(e){if(1!=arguments.length)throw x("atan",arguments.length,1);if(v(e))return Math.atan(e);if(e instanceof t){var r=e.re,i=e.im,a=r*r+(1-i)*(1-i),o=new t((1-i*i-r*r)/a,-2*r/a),f=Z.log(o);return new t(-.5*f.im,.5*f.re)}if(e instanceof Array||e instanceof n)return K.map(e,Z.atan);if(e.valueOf()!==e)return Z.atan(e.valueOf());throw O("atan",e)},Z.atan2=function(e,r){if(2!=arguments.length)throw x("atan2",arguments.length,2);if(v(e)){if(v(r))return Math.atan2(e,r)}else if(e instanceof t&&v(r))return Math.atan2(e.re,r);if(e instanceof Array||e instanceof n||r instanceof Array||r instanceof n)return K.map2(e,r,Z.atan2);if(r.valueOf()!==r||e.valueOf()!==e)return Z.atan2(e.valueOf(),r.valueOf());throw O("atan2",e,r)},Z.cos=function(e){if(1!=arguments.length)throw x("cos",arguments.length,1);if(v(e))return Math.cos(e);if(e instanceof t)return new t(.5*Math.cos(e.re)*(Math.exp(-e.im)+Math.exp(e.im)),.5*Math.sin(e.re)*(Math.exp(-e.im)-Math.exp(e.im)));if(e instanceof E){if(!e.hasBase(E.BASE_UNITS.ANGLE))throw new TypeError("Unit in function cos is no angle");return Math.cos(e.value)}if(e instanceof Array||e instanceof n)return K.map(e,Z.cos);if(e.valueOf()!==e)return Z.cos(e.valueOf());throw O("cos",e)},Z.cot=function(e){if(1!=arguments.length)throw x("cot",arguments.length,1);if(v(e))return 1/Math.tan(e);if(e instanceof t){var r=Math.exp(-4*e.im)-2*Math.exp(-2*e.im)*Math.cos(2*e.re)+1;return new t(2*Math.exp(-2*e.im)*Math.sin(2*e.re)/r,(Math.exp(-4*e.im)-1)/r)}if(e instanceof E){if(!e.hasBase(E.BASE_UNITS.ANGLE))throw new TypeError("Unit in function cot is no angle");return 1/Math.tan(e.value)}if(e instanceof Array||e instanceof n)return K.map(e,Z.cot);if(e.valueOf()!==e)return Z.cot(e.valueOf());throw O("cot",e)},Z.csc=function(e){if(1!=arguments.length)throw x("csc",arguments.length,1);if(v(e))return 1/Math.sin(e);if(e instanceof t){var r=.25*(Math.exp(-2*e.im)+Math.exp(2*e.im))-.5*Math.cos(2*e.re);return new t(.5*Math.sin(e.re)*(Math.exp(-e.im)+Math.exp(e.im))/r,.5*Math.cos(e.re)*(Math.exp(-e.im)-Math.exp(e.im))/r)}if(e instanceof E){if(!e.hasBase(E.BASE_UNITS.ANGLE))throw new TypeError("Unit in function csc is no angle");return 1/Math.sin(e.value)}if(e instanceof Array||e instanceof n)return K.map(e,Z.csc);if(e.valueOf()!==e)return Z.csc(e.valueOf());throw O("csc",e)},Z.sec=function(e){if(1!=arguments.length)throw x("sec",arguments.length,1);if(v(e))return 1/Math.cos(e);if(e instanceof t){var r=.25*(Math.exp(-2*e.im)+Math.exp(2*e.im))+.5*Math.cos(2*e.re);return new t(.5*Math.cos(e.re)*(Math.exp(-e.im)+Math.exp(e.im))/r,.5*Math.sin(e.re)*(Math.exp(e.im)-Math.exp(-e.im))/r)}if(e instanceof E){if(!e.hasBase(E.BASE_UNITS.ANGLE))throw new TypeError("Unit in function sec is no angle");return 1/Math.cos(e.value)}if(e instanceof Array||e instanceof n)return K.map(e,Z.sec);if(e.valueOf()!==e)return Z.sec(e.valueOf());throw O("sec",e)},Z.sin=function(e){if(1!=arguments.length)throw x("sin",arguments.length,1);if(v(e))return Math.sin(e);if(e instanceof t)return new t(.5*Math.sin(e.re)*(Math.exp(-e.im)+Math.exp(e.im)),.5*Math.cos(e.re)*(Math.exp(e.im)-Math.exp(-e.im)));if(e instanceof E){if(!e.hasBase(E.BASE_UNITS.ANGLE))throw new TypeError("Unit in function cos is no angle");return Math.sin(e.value)}if(e instanceof Array||e instanceof n)return K.map(e,Z.sin);if(e.valueOf()!==e)return Z.sin(e.valueOf());throw O("sin",e)},Z.tan=function(e){if(1!=arguments.length)throw x("tan",arguments.length,1);if(v(e))return Math.tan(e);if(e instanceof t){var r=Math.exp(-4*e.im)+2*Math.exp(-2*e.im)*Math.cos(2*e.re)+1;return new t(2*Math.exp(-2*e.im)*Math.sin(2*e.re)/r,(1-Math.exp(-4*e.im))/r)}if(e instanceof E){if(!e.hasBase(E.BASE_UNITS.ANGLE))throw new TypeError("Unit in function tan is no angle");return Math.tan(e.value)}if(e instanceof Array||e instanceof n)return K.map(e,Z.tan);if(e.valueOf()!==e)return Z.tan(e.valueOf());throw O("tan",e)},Z["in"]=function(e,t){if(2!=arguments.length)throw x("in",arguments.length,2);if(e instanceof E&&(t instanceof E||y(t)))return e.in(t);if(e instanceof Array||e instanceof n||t instanceof Array||t instanceof n)return K.map2(e,t,Z["in"]);if(e.valueOf()!==e||t.valueOf()!==t)return Z["in"](e.valueOf(),t.valueOf());throw O("in",e,t)},Z.clone=function tt(t){if(1!=arguments.length)throw x("clone",arguments.length,1);if(null==t)return t;if("function"==typeof t.clone)return t.clone();if(v(t)||y(t)||e(t))return t;if(t instanceof Array){var n=Z.clone;return t.map(function(e){return n(e)})}if(t instanceof Object)return K.mapObject(t,Z.clone);throw O("clone",t)},Z.eval=function(e){if(1!=arguments.length)throw x("eval",arguments.length,1);if(y(e))return nt.eval(e);if(e instanceof Array||e instanceof n)return K.map(e,Z.eval);throw new TypeError("String or matrix expected")};var nt=new Z.expr.Parser({readonly:!0});Z.format=function(e,t){var n=arguments.length;if(1!=n&&2!=n)throw x("format",n,1,2);if(1==n){var r=arguments[0];return v(r)?K.formatNumber(r):r instanceof Array?K.formatArray(r):y(r)?'"'+r+'"':r instanceof Object?""+r:r+""}if(!y(e))throw new TypeError("String expected as first parameter in function format");if(!(t instanceof Object))throw new TypeError("Object expected as first parameter in function format");return e.replace(/\$([\w\.]+)/g,function(e,n){for(var r=n.split("."),i=t[r.shift()];r.length&&void 0!=i;){var a=r.shift();i=a?i[a]:i+"."}return void 0!=i?i:e})},Z["import"]=function(e,t){var n,r={override:!1};if(t&&t instanceof Object&&K.extend(r,t),y(e)){if("undefined"==typeof require)throw Error("Cannot load file: require not available.");var i=require(e);Z["import"](i)}else if(Y(e)){if(n=e.name,!n)throw Error("Cannot import an unnamed function or object");(r.override||void 0===Z[n])&&H(n,e)}else if(e instanceof Object)for(n in e)if(e.hasOwnProperty(n)){var a=e[n];Y(a)?(r.override||void 0===Z[n])&&H(n,a):Z["import"](a)}},Z.select=function(e){return new Z.type.Selector(e)},Z["typeof"]=function(e){if(1!=arguments.length)throw x("typeof",arguments.length,1);var t,n=typeof e;if("object"==n){if(null==e)return"null";if(e instanceof Boolean)return"boolean";if(e instanceof Number)return"number";if(e instanceof String)return"string";if(e instanceof Array)return"array";if(e instanceof Date)return"date";if(e.constructor){for(t in Z)if(Z.hasOwnProperty(t)&&e.constructor==Z[t])return t.toLowerCase();for(t in Z.type)if(Z.type.hasOwnProperty(t)&&e.constructor==Z.type[t])return t.toLowerCase();if(e.constructor.name)return e.constructor.name.toLowerCase()}}return n},Z.Complex=function(){W("new math.Complex()","math.complex()")},Z.Unit=function(){W("new math.Unit()","math.unit()")},Z.parser.Parser=function(){W("new math.parser.Parser()","math.parser()")},Z.parser.Workspace=function(){W("new math.parser.Workspace()","math.workspace()")};for(var rt in Z)Z.hasOwnProperty(rt)&&rt&&d(rt,Z[rt])})();